#! /usr/bin/env python
# -*- coding=utf-8 -*-
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Subprocess.py'
__create_time__ = '2021/1/4 17:13'

import subprocess
import shlex
import re
import os
from datetime import datetime
from xiaobaiauto2.utils import _get_xpath, del_temp
from xiaobaiauto2.utils.code_template import *

devices = []
version = '0.0'
info = '''
# 本脚本为录制自动生成，本脚本会有部分的脚本与实际场景存在出入，例如：如何获取哪个操作是输入框的动作，哪个操作是按钮的操作，所以
# 导致无法准备生成输入的输入操作（send_keys方法），会在后续的版本中更新:）
# 脚本的自动生成属于人为操作与机器识别，故准确度不能保证为100%，解释权为小白科技拥有:)
# 未经许可不能用于商业用途！！！
# 可自由用于学习或者研究使用！！！
'''

def dataProcess(data=None, filename=''):
    if '' != filename and data:
        name, extname = os.path.splitext(filename)
        with open(name + '.py', 'a', encoding='utf-8') as f:
            if data.count('\t') == 2:
                ''' head '''
                data_list = data.split('\t')
                package_activity = data_list[2].split('/')
                code = appium_head_template_code % (info, version, data_list[1].replace('[', '').replace(']', ''),
                                                    package_activity[0], package_activity[1])
            elif data.count('\t') == 3:
                data_list = data.split('\t')
                if data_list[3] in ['//', None, 'None']:
                    code = appium_location_by_position_template_code % (data_list[2].replace('[', '').replace(']', ''),
                                                                        data_list[2].replace('[', '').replace(']', ''))
                else:
                    path_list = data_list[3].split('/')
                    if 'EditText' not in path_list[-1]:
                        code = appium_location_by_xpath_of_click_template_code % data_list[3]
                    else:
                        code = appium_location_by_xpath_of_sendkeys_template_code % data_list[3]
            else:
                code = ''
            f.write(code)
            f.flush()
            f.close()
    elif data:
        print(data)
    else:
        pass

def syncSub(cmd=None, match=None, other=None, deviceName=None, filename=''):
    if cmd:
        global devices, version
        all_r = []
        p = subprocess.Popen(shlex.split(cmd), shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while p.poll() is None:
            l = p.stdout.readline()
            l = l.strip()
            if l and match and other == 'getevent':
                r = [int(v, 16) for v in re.findall(match, str(l, encoding='utf-8'))]
                if r != [] and r != b'' and r != [0]:
                    all_r.extend(r)
            elif l and match and other:
                r = re.findall(match, str(l, encoding='utf-8'))
                if r != [] and r != b'' and r != [0]:
                    all_r.extend(r)
            if other == 'device':
                devices.extend(all_r)
                all_r.clear()
            elif other == 'version':
                if l != b'': version = l.decode('utf-8')
            elif other == 'activity':
                if all_r != []:
                    dataProcess(f'[{datetime.now()}]\t[{str(deviceName).split(" ")[-1]}]\t{all_r[0]}', filename)
                    all_r.clear()
            elif len(all_r) > 1:
                if other == 'getevent':
                    try:
                        print('[debug]-all_r:', all_r)
                        xpath = _get_xpath(x=all_r[0], y=all_r[1], deviceName=deviceName)
                        del_temp()
                        dataProcess(f'[{datetime.now()}]\t[{str(deviceName).split(" ")[-1]}]\t{all_r}\t{xpath}', filename)
                    except Exception as e:
                        dataProcess(f'[{datetime.now()}]\t[{str(deviceName).split(" ")[-1]}]\t{all_r}\t{e}')
                elif other == 'size':
                    dataProcess(f'[{datetime.now()}]\t[{str(deviceName).split(" ")[-1]}]\t{all_r}')
                all_r.clear()
        if p.returncode == 0:
            return True
        else:
            return False


#! /usr/bin/env python
# -*- coding=utf-8 -*-
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'template_code.py'
__create_time__ = '2020/11/11 19:19'

yaml_template_code = '''\
# 本数据仅限于Python语言的PyYaml库正常解析，不保证其它库可正常解析
public:
  version: 1.0
  site: &site http://test.xiaobai.com/api/
  env:
    token: 0
list:
  - title: 登录
    method: POST
    headers:
      Content-Type: application/x-www-form-data
    url: !!python/object/apply:os.path.join [*site, login]
    paramers:
    data: username=username&password=password
    assert_mode: json
    assert_path: code
    assert_value: 200
    extract_mode: json
    extract_at: body
    extract_path: data.token
    extract_name: token
  - title: 注册
    method: POST
    url: !!python/object/apply:os.path.join [*site, register]
    paramers:
    data: username=username&password=password&repassword=password
    assert_mode: json
    assert_path: code
    assert_value: 200
  - title: 列表
    method: GET
    headers:
      token: ${token}
    url: !!python/object/apply:os.path.join [*site, list]
    assert_mode: json
    assert_path: code
    assert_value: 200
    extract_mode: json
    extract_at: body
    extract_path: data[1].id
    extract_name: id
'''

appium_head_template_code = '''#! /usr/bin/env python
%s
from appium import webdriver
from selenium.webdriver.common.by import By

caps = {
    'automationName': 'uiautomator2',
    'platformName': 'Android',
    'platformVersion': '%s',
    'deviceName': '%s',
    'noReset': True,
    'allowClearUserData': 'true',
    'fullReset': "false",
    'exported': "true",
    'appPackage': '%s',
    'appActivity': '%s',
    'unicodeKeyboard': True,
    'resetKeyboard': True
}

app = webdriver.Remote('http://127.0.0.1:4723/wd/hub', caps)
# 点击动作可以录制，输入动作需要手动修改
'''

appium_location_by_xpath_of_click_template_code = '''
# xpath定位
app.find_element(by=By.XPATH, value='%s').click()
'''

appium_location_by_xpath_of_sendkeys_template_code = '''
# xpath定位
app.find_element(by=By.XPATH, value='%s').send_keys('#输入的内容#')
'''

appium_location_by_position_template_code = '''
# 点击坐标
app.swipe(%s, %s, 500)
'''



#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Help.py'
__create_time__ = '2020/7/22 20:10'

from tkinter import *
from tkinter.ttk import Notebook
from xiaobaiauto2.config.config import TIMERCONFIG
from xiaobaiauto2.utils.help_ui import HELP_UI
from xiaobaiauto2.__version__ import __version__

_timer = TIMERCONFIG()
_help_ui = HELP_UI()

root = Tk()
root.geometry('800x600')
root.title('小白工具箱 v%s' % __version__)
root.iconbitmap(_timer.timerpath)

tab = Notebook(root)
case_frame = Frame(master=tab)
vent_frame = Frame(master=tab)
option_frame = Frame(master=tab)
tab.add(case_frame, text='用例编辑')
tab.add(vent_frame, text='环境监测')
tab.add(option_frame, text='数据配置')
tab.pack(expand=True, fill=BOTH)

# HELP_UI(master=case_frame)
_help_ui.edit_case_ui(case_frame)
# _help_ui.db_view(option_frame)
root.mainloop()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Tools.py'
__create_time__ = '2020/9/23 23:14'

from typing import Optional
import argparse
import os
import zipfile
import sys
import shutil
from xiaobaiauto2.__version__ import __version__
from xiaobaiauto2.utils.jmeter_script_template import create_jmeter
from xiaobaiauto2.utils.xiaobaiauto2Installer import \
    jdk_install, jmeter_install, chromedriver_download, jenkins_war_install, git_windows_install, node_install,\
    allure_install, svn_install, adb_install, fiddler_install, postman_install
from jmespath import search
from urllib.parse import urlparse
from json import loads, JSONDecodeError
import json

step = '\\' if os.name == 'nt' else '/'

def raw_handle(s: Optional[str] = ''):
    '''
    数据分割处理（单个请求放在一起）
    :param s:
    :return:
    '''
    s = s.strip()
    sLines = s.split('\n')
    _start_index = [i for i, _ in enumerate(sLines) if 'HTTP/' in _ or ':authority:' in _]
    _span_data = []
    for i, v in enumerate(_start_index):
        if v != _start_index[-1]:
            _span_data.append(sLines[v:_start_index[i+1]])
        else:
            _span_data.append(sLines[v:])
    # 数据分离(method、url、headers、data)
    result = []
    for request in _span_data:
        if 'HTTP/' in request[0]:
            _method = request[0].split(' ')[0]
            _headers = {}
            _headers_end = 0
            raw_header_data_list = request[1:]
            for _, j in enumerate(raw_header_data_list):
                _headers_end = _
                if '' == j:
                    break
                else:
                    _headers[j.split(': ')[0]] = j.split(': ')[1].strip()
            if raw_header_data_list.__len__() > _headers_end + 1:
                _data = ''.join([i for i in request[_headers_end + 1:] if i != ''])
            else:
                _data = ''
            _url = request[0].split(' ')[1]
            if '://' not in _url and '443' in _headers.get('Host'):
                _url = 'https://' + _headers.get('Host') + _url
            elif '://' not in _url and '443' not in _headers.get('Host'):
                _url = 'http://' + _headers.get('Host') + _url
            result.append(
                {
                    'method': _method,
                    'url': _url,
                    'headers': _headers,
                    'data': _data
                }
            )
        elif ':authority:' in request[0]:
            '''
            :authority: 域名
            :method:    方式
            :path:      地址
            :scheme:    协议
            '''
            _authority = request[0].split(':')[2].strip()
            _method = request[1].split(':')[2].strip()
            _path = request[2].split(':')[2].strip()
            _scheme = request[3].split(':')[2].strip()
            _url = _scheme + '://' + _authority + _path
            _headers = {}
            _headers_end = 0
            raw_header_data_list = request[4:]
            for _, j in enumerate(raw_header_data_list):
                _headers_end = _
                if '' == j:
                    break
                else:
                    _headers[j.split(': ')[0]] = j.split(': ')[1].strip()
            if raw_header_data_list.__len__() > _headers_end + 1:
                _data = ''.join([i for i in request[_headers_end + 1:] if i != ''])
            else:
                _data = ''
            result.append(
                {
                    'method': _method,
                    'url': _url,
                    'headers': _headers,
                    'data': _data
                }
            )
    return result

def har_handle(s: Optional[str] = ''):
    '''
    将har文件字符串，提取接口数据
    :param s:
    :return:
    '''
    result = []
    try:
        d = loads(s)
    except json.decoder.JSONDecodeError as e:
        if s.startswith('\ufeff'):
            s = s.encode('utf8')[3:].decode('utf8')
            d = loads(s)
        else:
            result.append({
                'method': '',
                'url': '',
                'headers': {},
                'data': '',
                'response': None
            })
            return result
    except JSONDecodeError as e:
        d = loads(s.replace("'", '"'))
    except ValueError as e:
        result.append({
            'method': '',
            'url': '',
            'headers': {},
            'data': '',
            'response': None
        })
        return result
    search_request = search('log.entries[].request', d)
    search_response = search('log.entries[].response', d)
    for i, request in enumerate(search_request):
        _method = request.get('method')
        _url = request.get('url')
        _headers = {}
        _data = ''
        if 'content' in search_response[i].keys() and 'mimeType' in search_response[i]['content'].keys() and \
            'text' in search_response[i]['content']:
                _response = search_response[i]['content']['text']
        else:
            _response = None
        for item in request.get('headers'):
            if item.get('name') not in [':method', ':path', ':authority', ':scheme']:
                _headers[item.get('name')] = item.get('value')
        if _method.upper() != 'GET':
            if 'json' in request.get('postData').get('mimeType'):
                if 'text' in request['postData'].keys():  # browser(chrome)
                    _data = request['postData']['text']
                elif 'params' in request['postData'].keys():  # fiddler/charles
                    _data = {}
                    for item in request.get('postData').get('params'):
                        _data[item.get('name')] = item.get('value')
                    _data = _data.__str__().replace("'", '"')
            else:
                if 'text' in request['postData'].keys():  # browser(chrome)
                    _data = request['postData']['text']
                elif 'params' in request['postData'].keys():  # fiddler/charles
                    try:
                        for item in request.get('postData').get('params'):
                            _data += item.get('name') + '=' + item.get('value') + '&'
                        _data = _data[:-1]
                    except TypeError as e:
                        try:
                            _data = request.get('postData').get('params')[0].get('value')
                        except TypeError as e:
                            pass
        result.append(
            {
                'method': _method,
                'url': _url,
                'headers': _headers,
                'data': _data,
                'response': _response
            }
        )
    return result

def convert(data: Optional[list], is_xiaobaiauto2: Optional[int] = 0) -> str:
    '''
    代码转换器
    :param data: {'method':'', 'url': '', 'headers': '', 'data': ''}
    :param is_xiaobaiauto2: 是否转为xiaobaiauto2库
    :param is_har: 是否是har文件
    :return:
    '''
    _code_top = '#! /usr/bin/env python\n'
    _code_pytest_import = \
        'from re import findall\n' \
        'try:\n\timport pytest\n' \
        '\timport requests\n' \
        'except ModuleNotFoundError as e:\n' \
        '\timport os\n' \
        '\tos.system("pip install xiaobaiauto2")\n' \
        '\timport pytest\n' \
        '\timport requests\n\n' \
        'def setup_function():\n' \
        '\t""" 每个接口执行前都会执行的方法，存放公共变量、加密等操作 """\n' \
        '\tpass\n\n' \
        'def teardown_function():\n' \
        '\t""" 每个接口执行后都会执行的方法，存放断言、提取值等操作 """\n' \
        '\tpass\n\n'
    _code_xiaobaiauto2_import = \
        'from os import path\n' \
        'try:\n\timport pytest\n' \
        '\tfrom xiaobaiauto2.xiaobaiauto2 import ' \
        'api_action, assert_mode, extract_at, extract_mode, PUBLIC_VARS, sleep\n' \
        '\tfrom xiaobaiauto2.utils.xiaobaiauto2Email import send_email\n' \
        '\tfrom xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS\n' \
        '\tfrom xiaobaiauto2.config.config import EMAILCONFIG\n' \
        'except ModuleNotFoundError as e:\n' \
        '\timport os\n' \
        '\tos.system("pip install xiaobaiauto2")\n' \
        '\timport pytest\n' \
        '\tfrom xiaobaiauto2.xiaobaiauto2 import ' \
        'api_action, assert_mode, extract_at, extract_mode, PUBLIC_VARS, sleep\n' \
        '\tfrom xiaobaiauto2.utils.xiaobaiauto2Email import send_email\n' \
        '\tfrom xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS\n' \
        '\tfrom xiaobaiauto2.config.config import EMAILCONFIG\n\n'
    _code_requests_import = \
        'from re import findall\n' \
        'try:\n\timport requests\n' \
        'except ModuleNotFoundError as e:\n' \
        '\timport os\n' \
        '\tos.system("pip install xiaobaiauto2")\n' \
        '\timport requests\n\n'
    _code_pytest_end = \
        '# 脚本命令行运行须知： ' \
        '\r# pytest -s -v   运行当前目录所有test_*开头的脚本文件' \
        '\r# pytest -s -v xxx.py 运行指定脚本文件' \
        '\r# pytest -s -v --html=report.html  运行并将结果记录到HTML报告中' \
        '\r# pytest -s -v --html=report.html --self-contained-html 运行并将结果记录到HTML报告中' \
        '\r# pytest其他运行方式参考https://pypi.org/project/xiaobaiauto2或官网说明'
    _code_xiaobaiauto2_end = \
        '@pytest.mark.last\n' \
        'def test_last():\n' \
        '\t#print("测试结束了，发个邮件吧")\n' \
        '\tsleep(2)\n' \
        '\t_emil = EMAILCONFIG()\n' \
        '\t_cur_path = path.abspath(path.curdir)\n' \
        '\tPUBLIC_VARS["report"] = "report.html"  # 命令行您期望的报告文件名\n' \
        '\tif "report" in PUBLIC_VARS.keys() and "" != PUBLIC_VARS["report"]:\n' \
        '\t\tif path.isfile(_cur_path + "/" + PUBLIC_VARS["report"]):\n' \
        '\t\t\tsend_email(_cur_path + "/" + PUBLIC_VARS["report"])\n' \
        '\telif "" != _emil.report:\n' \
        '\t\tif path.isfile(_cur_path + "/" + _emil.report):\n' \
        '\t\t\tsend_email(_cur_path + "/" + _emil.report)\n\n' \
        '@pytest.mark.first\n' \
        'def test_first():\n' \
        '\t# print("测试开始了，准备邮件信息")\n' \
        '\temil = {\n' \
        '\t\t"sender": "807447312@qq.com",\n' \
        '\t\t"receiver": ["807447312@qq.com", "qiankuny@163.com", "912194099@qq.com"],\n' \
        '\t\t"smtpserver": "smtp.163.com",\n' \
        '\t\t"smtp_port": 25,\n' \
        '\t\t"username": "807447312@qq.com",\n' \
        '\t\t"password": "",\n' \
        '\t\t"subject": "小白自动化测试报告",\n' \
        '\t\t"report": "report.html"\n' \
        '\t}\n' \
        '\tPUBLIC_VARS.update(emil)\n\n'
    _code = ''
    for i, v in enumerate(data):
        if is_xiaobaiauto2 == 1:
            _code += f'''@pytest.mark.run(order={i + 2})\
                \rdef test_xiaobai_api_{i + 1}():\
                \r\t# 测试前数据准备 其中下方代码中{{变量名}}是需要在前面的接口返回值提取 \
                \r\theaders = {v.get('headers')}\
                \r\turl = '{v.get('url')}' \
                \r\tdata = '{v.get('data')}'.encode('utf-8')\
                \r\tresponse = requests.request(method='{v.get('method')}', url=url, data=data, headers=headers)\
                \r\t# 测试后时间判断/提取\
                \r\t# assert response.status_code == 200  # 判断HTTP响应状态\
                \r\t# var_name = response.headers()[路径]  # 提取响应头数据\
                \r\t# global var_name # 设置全局变量\
                \r\tif 'json' in response.headers.get('Content-Type'):\
                \r\t\t# assert '预期结果' == response.json()[路径]  # 判断json响应体结果\
                \r\t\t# var_name = response.json()[路径]  # 提取json响应体数据\
                \r\t\t# var_name = response.headers()[路径]  # 提取响应头数据\
                \r\t\tprint(response.json())\
                \r\telse:\
                \r\t\t# assert '预期结果' in response.text # 判断字符串返回值结果 \
                \r\t\t# var_name = findall('正则表达式', response.text)[0] # 正则提取数据\
                \r\t\tprint(response.text)\n\n'''
        elif is_xiaobaiauto2 == 2:
            _code += f'''@pytest.mark.run(order={i + 2})\
                \rdef test_xiaobai_api_{i + 1}():\
                \r\t# 测试前数据准备\
                \r\theaders = {v.get('headers')}\
                \r\turl = '{v.get('url')}' \
                \r\tdata = '{v.get('data')}'.encode('utf-8')\
                \r\tapi_action(\
                \r\t\tmethod='{v.get('method')}',\
                \r\t\turl=url,\
                \r\t\tdata=data,\
                \r\t\theaders=headers,\
                \r\t\tassert_mode=assert_mode.json,\
                \r\t\tassert_path='',\
                \r\t\tassert_value='',\
                \r\t\textract_at=extract_at.body,\
                \r\t\textract_mode=extract_mode.json,\
                \r\t\textract_path='',\
                \r\t\textract_name=''\
                \r\t)\
                \r\t#:param method: 接口请求方式\
                \r\t#:param url: 接口请求地址\
                \r\t#:param data: 接口请求数据\
                \r\t#:param headers: 接口请求头\
                \r\t# 其他可用参数如下：\
                \r\t#:param assert_mode: 校验模式[json, contains]\
                \r\t#:param assert_path:  校验预期结果路径\
                \r\t#:param assert_value: 校验预期结果\
                \r\t#:param extract_at:  提取数据所在位置 [status, headers, body, all]\
                \r\t#:param extract_mode: 提取方式 [json, re]\
                \r\t#:param extract_path: 提取路径表达式\
                \r\t#:param extract_name: 提取数据变量名\
                \r\t#:param kwargs: 其他参数参考requests的api\n\n'''
        else:
            _code += f'''\r# 测试前数据准备 其中下方代码中{{变量名}}是需要在前面的接口返回值提取 \
                \rheaders = {v.get('headers')}\
                \rurl = '{v.get('url')}' \
                \rdata = '{v.get('data')}'.encode('utf-8')\
                \rresponse = requests.request(method='{v.get('method')}', url=url, data=data, headers=headers)\
                \r# 测试后数据判断/提取\
                \r# assert response.status_code == 200  # 判断HTTP响应状态\
                \r# var_name = response.headers()[路径]  # 提取响应头数据\
                \rif 'json' in response.headers.get('Content-Type'):\
                \r\t# assert '预期结果' == response.json()[路径]  # 判断json响应体结果\
                \r\t# var_name = response.json()[路径]  # 提取json响应体数据\
                \r\tprint(response.json())\
                \relse:\
                \r\t# assert '预期结果' in response.text # 判断字符串返回值结果 \
                \r\t# var_name = findall('正则表达式', response.text)[0] # 正则提取数据\
                \r\tprint(response.text)\n\n'''
    if is_xiaobaiauto2 == 1:
        return _code_top + _code_pytest_import + _code + _code_pytest_end
    elif is_xiaobaiauto2 == 2:
        return _code_top + _code_xiaobaiauto2_import + _code + _code_xiaobaiauto2_end + \
               '\n# 使用公共变量的格式：\n# PUBLIC_VARS["变量名"]\n' + _code_pytest_end
    elif is_xiaobaiauto2 == 3:
        return create_jmeter(request=data)
    else:
        return _code_top + _code_requests_import + _code

def har_convert(data: Optional[str], is_xiaobaiauto2: Optional[int] = 0):
    '''
    har单文件转换
    :param data:
    :param is_xiaobaiauto2:
    :return:
    '''
    if data != '':
        return convert(data=har_handle(data), is_xiaobaiauto2=is_xiaobaiauto2)
    else:
        return ''

def raw_convert(raw: Optional[str], is_xiaobaiauto2: Optional[int] = 0) -> str:
    '''
    原文转换器
    :param raw:
    :param is_xiaobaiauto2: 是否转xiaobaiauto2库代码样例
    :return:
    '''
    if raw != '':
        return convert(data=raw_handle(raw), is_xiaobaiauto2=is_xiaobaiauto2)
    else:
        return ''

def file_convert(data: Optional[str], is_xiaobaiauto2: Optional[int], is_har: Optional[bool]):
    if is_har:
        return har_convert(data=data, is_xiaobaiauto2=is_xiaobaiauto2)
    else:
        return raw_convert(raw=data, is_xiaobaiauto2=is_xiaobaiauto2)

def compare_str(k: Optional[str] = None, s0: Optional[str] = '', s1: Optional[str] = '',
                t: Optional[str] = 'text', is_xiaobaiauto2: Optional[int] = 0):
    '''
    比较两个字符串区别，暂时只考虑请求数据的不同，其他情况暂时忽略
    data = 'a=1&b=2&c=3'  字符串型
    data = '{'a':1, 'b': '0'}'  字典型
    data = '！@#￥%……&*'         文件内容型==字符串型
    :param s0: 字符串1
    :param s1: 字符串2
    :param t:  字符串是否含请求头
    :return:
    '''
    if 3 == is_xiaobaiauto2:
        start_step = '${'
        stop_step = '}'
    elif 4 == is_xiaobaiauto2:
        start_step = '{{'
        stop_step = '}}'
    else:
        start_step = '{'
        stop_step = '}'
    if 'json' in t and '{' in s0 and '}' in s0 and ':' in s0:
        try:
            d0 = loads(s0)
            d1 = loads(s1)
        except json.decoder.JSONDecodeError as e:
            try:
                if s0.startswith('\ufeff'):
                    s0 = s0.encode('utf8')[3:].decode('utf8')
                    d0 = loads(s0)
                if s1.startswith('\ufeff'):
                    s1 = s1.encode('utf8')[3:].decode('utf8')
                    d1 = loads(s1)
            except Exception as e:
                pass
        except JSONDecodeError as e:
            d0 = loads(s0.replace("'", '"'))
            d1 = loads(s1.replace("'", '"'))
        finally:
            return compare_dict(d0, d1, is_xiaobaiauto2=is_xiaobaiauto2)
    elif isinstance(s0, dict) and isinstance(s1, dict):
        compare_dict(s0, s1, is_xiaobaiauto2=is_xiaobaiauto2)
    else:
        IS_CUT = False
        result_str = ''
        s0s = s0
        s1s = s1
        if '?' in s0 and '?' in s1:
            s0 = s0[s0.find('?') + 1:]
            s1 = s1[s1.find('?') + 1:]
            IS_CUT = True
        # 分割参数
        sl0 = s0.split('&')
        sl0.sort()
        sl1 = s1.split('&')
        sl1.sort()
        for s_0, s_1 in zip(sl0, sl1):
            if '=' in s_0 and '=' in s_1:
                s_0l = s_0.split('=')
                s_1l = s_1.split('=')
                if s_0l[0] == s_1l[0] and s_0l[1] != s_1l[1]:
                    result_str += s_0l[0] + '=' + start_step + s_0l[0] + stop_step + '&'
                else:
                    result_str += s_0 + '&'
            elif s0s != s1s:
                result_str += start_step + k + stop_step
        if '&' == result_str[-1]:
            if IS_CUT:
                return s0s[:s0s.find('?') + 1] + result_str[:-1]
            else:
                return result_str[:-1]
        else:
            if IS_CUT:
                return s0s[:s0s.find('?') + 1] + result_str
            else:
                return result_str

def compare_dict(d0: Optional[dict], d1: Optional[dict], is_xiaobaiauto2: Optional[int] = 0) -> dict:
    '''
    比较两个字典数据
    :param d0:  字典对象1
    :param d1:  字典对象2
    :return:
    '''
    if not isinstance(d0, dict) and not isinstance(d1, dict) or d0 == {} or d1 == {}:
        return {}
    diff = d0.keys() & d1
    diffr = [(k, d0[k], d1[k]) for k in diff if d0[k] != d1[k]]
    diff_api = False
    if diffr.__len__() > 0:
        for v in diffr:
            if v[0] == 'method' and d0['method'] != d1['method']:
                diff_api = True
                break
            elif v[0] == 'url' and urlparse(d0['url']).path != urlparse(d1['url']).path:
                diff_api = True
                break
            elif v[0] == 'headers':
                d0[v[0]] = compare_dict(v[1], v[2], is_xiaobaiauto2)
            else:
                if 'headers' in d0.keys() and 'content-type' in d0['headers'].keys():
                    d0[v[0]] = compare_str(k=v[0], s0=v[1], s1=v[2], t=d0['headers']['content-type'], is_xiaobaiauto2=is_xiaobaiauto2)
                elif 'headers' in d0.keys() and 'Content-Type' in d0['headers'].keys():
                    d0[v[0]] = compare_str(k=v[0], s0=v[1], s1=v[2], t=d0['headers']['Content-Type'], is_xiaobaiauto2=is_xiaobaiauto2)
                else:
                    d0[v[0]] = compare_str(k=v[0], s0=v[1], s1=v[2], t='text', is_xiaobaiauto2=is_xiaobaiauto2)
    if diff_api:
        return {}
    else:
        d1.update(d0)
        return d1

def compare_har_convert(s0: Optional[str], s1: Optional[str], is_xiaobaiauto2: Optional[int] = 0):
    '''
    比较两个har文件字符串
    :param s0:
    :param s1:
    :return:
    '''
    result0 = har_handle(s0)
    result1 = har_handle(s1)
    result = []
    # 两个脚本数据雷同，记录条数一致
    for r0, r1 in zip(result0, result1):
        r = compare_dict(d0=r0, d1=r1, is_xiaobaiauto2=is_xiaobaiauto2)
        if r != {}:
            result.append(r)
        else:
            result.append(r0)
            result.append(r1)
    if result0.__len__() < result1.__len__():
        result.extend(result1[result0.__len__():])
    elif result1.__len__() < result0.__len__():
        result.extend(result0[result1.__len__():])
    if result.__len__() != 0:
        return convert(data=result, is_xiaobaiauto2=is_xiaobaiauto2)
    else:
        return ''

def yaml_convert(data: Optional[dict], is_xiaobaiauto2: Optional[int] = 0) -> list:
    '''
    YAML文档转python代码
    :param data:
    :param is_xiaobaiauto2:
    :return:
    YAML样例：
        -----------------------------------------------------------------------
        |# 本数据仅限于Python语言的PyYaml库正常解析，不保证其它库可正常解析
        |public:
        |  version: 1.0
        |  site: &site http://test.xiaobai.com/api/
        |  env:
        |    token: 0
        |list:
        |  - title: 登录
        |    method: POST
        |    headers:
        |      Content-Type: application/x-www-form-data
        |    url: !!python/object/apply:os.path.join [*site, login]
        |    paramers:
        |    data: username=username&password=password
        |    assert_mode: json
        |    assert_path: code
        |    assert_value: 200
        |    extract_mode: json
        |    extract_at: body
        |    extract_path: data.token
        |    extract_name: token
        |  - title: 注册
        |    method: POST
        |    url: !!python/object/apply:os.path.join [*site, register]
        |    paramers:
        |    data: username=username&password=password&repassword=password
        |    assert_mode: json
        |    assert_path: code
        |    assert_value: 200
        |  - title: 列表
        |    method: GET
        |    headers:
        |      token: ${token}
        |    url: !!python/object/apply:os.path.join [*site, list]
        |    assert_mode: json
        |    assert_path: code
        |    assert_value: 200
        |    extract_mode: json
        |    extract_at: body
        |    extract_path: data[1].id
        |    extract_name: id
        ------------------------------------------------------------------------
        |样例解释：
        |    &site 设置的变量，供yaml下文调用
        |    *site 表示调用yaml中的变量
        |    env 表示环境，下面定义的都是代码运行中的临时变量
        |    list 表示接口列表
        |    !!python/object/apply:os.path.join [*site, login]
        |    上面的内容表示：将yaml变量与字符串连接，如果表示域名，变量末尾最好带“/”符合
        |    ${token} 表示动态获取环境中的变量
        -------------------------------------------------------------------------
    '''

def copy_template(source: str = '', target: str = '.'):
    CMD = f'xcopy /E/S "{source}" "{target}"' if os.name == 'nt' else f'cp -r {source} {target}'
    if os.path.isdir(target):
        os.popen(CMD)

def api_raw(c: Optional[int], f: Optional[str], d: Optional[str], s: Optional[str], x: Optional[int]):
    is_har = False
    if x == 3:
        extname = '.jmx'
    else:
        extname = '.py'
    if ',' not in f or c == 0:
        if f != '' and os.path.isfile(f):
            raw_data = ''
            if os.path.splitext(f)[1] == '.saz':
                is_har = False
                raw_file_path = os.path.splitext(f)[0]
                zipfile.ZipFile(f).extractall(raw_file_path)
                raw_file_list = [i for i in os.listdir(raw_file_path + step + 'raw') if '_c.txt' == i[-6:]]
                for i in raw_file_list:
                    with open(raw_file_path + step + 'raw' + step + i, 'r', encoding='utf-8') as fr:
                        raw_data += fr.read() + '\n\n\n'
                        fr.close()
                if os.path.isdir(raw_file_path):
                    try:
                        os.remove(raw_file_path)
                    except PermissionError as e:
                        pass
            elif os.path.splitext(f)[1] == '.har':
                is_har = True
                with open(f, 'r', encoding='utf-8') as fr:
                    raw_data += fr.read()
                    fr.close()
            elif os.path.splitext(f)[1] == '.txt':
                with open(f, 'r', encoding='utf-8') as fr:
                    is_har = False
                    raw_data += fr.read()
                    fr.close()
            code = file_convert(data=raw_data, is_xiaobaiauto2=x, is_har=is_har)
            if s != '':
                with open(s + extname, 'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
            else:
                with open(os.path.splitext(f)[0] + extname, 'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
        else:
            if f != '':
                if os.path.isfile(d + step + f):
                    raw_data = ''
                    if os.path.splitext(d + step + f)[1] == '.saz':
                        is_har = False
                        raw_file_path = os.path.splitext(d + step + f)[0]
                        zipfile.ZipFile(d + step + f).extractall(raw_file_path)
                        raw_file_list = [i for i in os.listdir(d + step + raw_file_path + step + 'raw') if '_c.txt' == i[-6:]]
                        for i in raw_file_list:
                            with open(d + step + raw_file_path + step + 'raw' + step + i, 'r') as fr:
                                raw_data += fr.read() + '\n\n\n'
                                fr.close()
                        if os.path.isdir(d + step + raw_file_path):
                            try:
                                os.remove(d + step + raw_file_path)
                            except PermissionError as e:
                                raise (e)
                    elif os.path.splitext(d + step + f)[1] == '.har':
                        is_har = True
                        with open(d + '/' + f, 'r', encoding='utf-8') as fr:
                            raw_data += fr.read()
                            fr.close()
                    elif os.path.splitext(d + step + f)[1] == '.txt':
                        with open(d + step + f, 'r', encoding='utf-8') as fr:
                            is_har = False
                            raw_data += fr.read()
                            fr.close()
                    code = file_convert(data=raw_data, is_xiaobaiauto2=x, is_har=is_har)
                    if s != '':
                        with open(d + step + s + extname, 'w', encoding='utf-8') as fw:
                            fw.write(code)
                            fw.flush()
                            fw.close()
                    else:
                        with open(os.path.splitext(d + step + f)[0] + extname, 'w', encoding='utf-8') as fw:
                            fw.write(code)
                            fw.flush()
                            fw.close()
            else:
                for f in [i for i in os.listdir(d) if os.path.splitext(i)[1] in ['.saz', '.har', '.txt']]:
                    raw_data = ''
                    if os.path.splitext(d + step + f)[1] == '.saz':
                        is_har = False
                        raw_file_path = os.path.splitext(d + step + f)[0]
                        zipfile.ZipFile(d + step + f).extractall(raw_file_path)
                        raw_file_list = [i for i in os.listdir(raw_file_path + step + 'raw') if
                                         '_c.txt' == i[-6:]]
                        for i in raw_file_list:
                            with open(raw_file_path + step + 'raw' + step + i, 'r') as fr:
                                raw_data += fr.read() + '\n\n\n'
                                fr.close()
                        if os.path.isdir(raw_file_path):
                            try:
                                os.remove(raw_file_path)
                            except PermissionError as e:
                                pass
                    elif os.path.splitext(d + step + f)[1] == '.har':
                        is_har = True
                        with open(d + step + f, 'r', encoding='utf-8') as fr:
                            raw_data += fr.read()
                            fr.close()
                    elif os.path.splitext(d + step + f)[1] == '.txt':
                        with open(d + step + f, 'r', encoding='utf-8') as fr:
                            is_har = False
                            raw_data += fr.read()
                            fr.close()
                    code = file_convert(data=raw_data, is_xiaobaiauto2=x, is_har=is_har)
                    if s != '':
                        with open(d + step + s + extname, 'w', encoding='utf-8') as fw:
                            fw.write(code)
                            fw.flush()
                            fw.close()
                    else:
                        with open(os.path.splitext(d + step + f)[0] + extname, 'w', encoding='utf-8') as fw:
                            fw.write(code)
                            fw.flush()
                            fw.close()
    elif ',' in f and c == 1:
        f_l = f.split(',')
        if f_l.__len__() == 2 and \
                os.path.isfile(f_l[0]) and \
                os.path.isfile(f_l[1]) and \
                os.path.splitext(f_l[0])[1] == '.har':
            ''' 暂时支持比较 *.har格式文件 '''
            data0 = ''
            data1 = ''
            with open(f_l[0], 'r', encoding='utf-8') as fr:
                data0 += fr.read()
                fr.close()
            with open(f_l[1], 'r', encoding='utf-8') as fr:
                data1 += fr.read()
                fr.close()
            code = compare_har_convert(s0=data0, s1=data1, is_xiaobaiauto2=x)
            if s != '':
                with open(s + extname, 'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
            else:
                with open(os.path.splitext(f_l[0])[0] + '_' + os.path.splitext(f_l[1])[0] + '_compare' + extname,
                          'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
        elif f_l.__len__() == 2 and \
                os.path.isfile(d + step + f_l[0]) and \
                os.path.isfile(d + step + f_l[1]) and \
                os.path.splitext(f_l[0])[1] == '.har':
            ''' 暂时支持比较 *.har格式文件 '''
            data0 = ''
            data1 = ''
            with open(d + step + f_l[0], 'r', encoding='utf-8') as fr:
                data0 += fr.read()
                fr.close()
            with open(d + step + f_l[1], 'r', encoding='utf-8') as fr:
                data1 += fr.read()
                fr.close()
            code = compare_har_convert(s0=data0, s1=data1, is_xiaobaiauto2=x)
            if s != '':
                with open(d + step + s + extname, 'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
            else:
                with open(d + step + os.path.splitext(f_l[0])[0] + '_' + os.path.splitext(f_l[1])[0] + '_compare' + extname,
                          'w', encoding='utf-8') as fw:
                    fw.write(code)
                    fw.flush()
                    fw.close()
        else:
            ''' 大于两个文件比较 '''
            print('目前支持两个文件比较，两次的请求包已经可以比较出其差异:>')

def cmd():
    arg = argparse.ArgumentParser(
        '小白科技·Python接口转换器·工具下载器 v' + __version__
    )
    arg.add_argument('-c', '--compare',
                     type=int,
                     choices=(0, 1),
                     default=0,
                     help='比较两个har文件的区别,并转为python代码,0:不比较(默认),1:比较')
    arg.add_argument('-f', '--file', type=str, default='', help='支持har|saz|txt扩展名的raw数据文件')
    arg.add_argument('-d', '--dir', type=str, default='.',
                     help='批量转换指定目录下所有har|saz|txt扩展名的raw数据文件, 默认当前目录')
    arg.add_argument('-s', '--save', type=str, default='', help='默认生成同名的脚本文件,省略扩展名')
    arg.add_argument('-x', '--xiaobai', type=int, choices=(0, 1, 2, 3), default=0,
                     help='0:requests格式(默认),1:pytest格式,2:xiaobaiauto2格式,3:jmeter格式')
    arg.add_argument('-t', '--template', default='',
                     help='获取模板到本地,无模板(默认),api模板,web模板,app模板')
    arg.add_argument('-i', '--install', default=-1, help='(默认)不安装,支持jdk|jmeter|chromdriver|jenkins|git|node')
    arg.add_argument('-v', '--version', type=str, default='latest', help='指定软件的版本，默认最新版')
    params = arg.parse_args()
    package_path_list = [p for p in sys.path if '\\lib\\site-packages' in p]
    if sys.platform == 'win32':
        package_path = package_path_list[0] \
            if len(package_path_list) > 0 else os.popen('pip show xiaobaiauto2').readlines()[7].split(' ')[-1][:-1]
    else:
        package_path = package_path_list[0] \
            if len(package_path_list) > 0 else os.popen('pip3 show xiaobaiauto2').readlines()[7].split(' ')[-1][:-1]
    if params.install in [0, 'jdk']:
        v = params.version
        if v != '':
            jdk_install(v, params.dir)
        else:
            jdk_install(dest_dir=params.dir)
    elif params.install in [1, 'jmeter', 'JMeter']:
        v = params.version
        if v != '':
            jmeter_install(v, params.dir)
        else:
            jmeter_install(dest_dir=params.dir)
    elif params.install in [2, 'chromedriver', 'ChromeDriver']:
        v = params.version
        if v != '':
            chromedriver_download(v, params.dir)
        else:
            chromedriver_download(dest_dir=params.dir)
    elif params.install == 'jenkins':
        v = params.version
        if v != '':
            jenkins_war_install(v, params.dir)
        else:
            jenkins_war_install(dest_dir=params.dir)
    elif params.install == 'git':
        v = params.version
        if v != '':
            git_windows_install(v, params.dir)
        else:
            git_windows_install(dest_dir=params.dir)
    elif params.install in ['node', 'nodejs']:
        v = params.version
        if v != '':
            node_install(v, params.dir)
        else:
            node_install(dest_dir=params.dir)
    elif params.install in ['allure', 'Allure']:
        v = params.version
        if v != '':
            allure_install(v, params.dir)
        else:
            allure_install(dest_dir=params.dir)
    elif params.install in ['svn', 'subversion']:
        v = params.version
        if v != '':
            svn_install(v, params.dir)
        else:
            svn_install(dest_dir=params.dir)
    elif params.install in ['adb']:
        v = params.version
        if v != '':
            adb_install(v, params.dir)
        else:
            adb_install(dest_dir=params.dir)
    elif params.install in ['fiddler', 'Fiddler']:
        v = params.version
        if v != '':
            fiddler_install(v, params.dir)
        else:
            fiddler_install(dest_dir=params.dir)
    elif params.install in ['postman', 'PostMan']:
        v = params.version
        if v != '':
            postman_install(v, params.dir)
        else:
            postman_install(dest_dir=params.dir)
    elif params.template in [0, 'api']:
        shutil.copytree(f'{package_path}{step}xiaobaiauto2{step}test{step}ApiTest', f'{params.dir}{step}ApiTest')
    elif params.template in [1, 'web']:
        shutil.copytree(f'{package_path}{step}xiaobaiauto2{step}test{step}WebTest', f'{params.dir}{step}WebTest')
    elif params.template in [2, 'app']:
        shutil.copytree(f'{package_path}{step}xiaobaiauto2{step}test{step}AppTest', f'{params.dir}{step}AppTest')
    else:
        api_raw(c=params.compare, f=params.file, d=params.dir, s=params.save, x=params.xiaobai)


#! /usr/bin/PUBLIC_VARS python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test_case1.py'
__create_time__ = '2020/10/4 21:44'

from .yewua import *
import pytest

PUBLIC_VARS['base'] = 'https://test.xiaobaisoftware.com'  # 测试环境接口域名

@pytest.mark.yewua
def test_yewu():
    ''' 测试业务接口 '''

    post_api_user_login(
        data='phone=16015234678&password=123456',
        assert_mode=assert_mode.json,
        assert_path='code',
        assert_value=100,
        extract_at=extract_at.body,
        extract_mode=extract_mode.json,
        extract_path='data.token',
        extract_name='token'
    )
    post_api_getList(
        data='page=1&pageSize=10',
        assert_mode=assert_mode.json,
        assert_path='code',
        assert_value=100,
        extract_at=extract_at.body,
        extract_mode=extract_mode.json,
        extract_path='data[0].id',
        extract_name='id'
    )
    post_api_getInfo(
        data='id=' + PUBLIC_VARS.get('id'),
        headers={'token': PUBLIC_VARS.get('token')},
        assert_mode=assert_mode.json,
        assert_path='code',
        assert_value=100
    )
    post_api_saveOrder(
        data=f'id={PUBLIC_VARS.get("id")}&receiveName=11&phone=22&wxNumber=33',
        headers={'token': PUBLIC_VARS.get('token')},
        assert_mode=assert_mode.json,
        assert_path='code',
        assert_value=100
    )


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'conftest.py'
__create_time__ = '2020/7/6 14:11'
'''  UI自动化可以使用该文件，其他测试可以移除防止对测试有干扰   '''
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS
from xiaobaiauto2.utils.xiaobaiauto2Installer import chromedriver_download
import pytest

b = None

@pytest.mark.hookwrapper
def pytest_runtest_makereport(item):
    """
    当测试失败的时候，自动截图，展示到html报告中
    :param item:
    """
    pytest_html = item.config.pluginmanager.getplugin('html')
    outcome = yield
    report = outcome.get_result()
    extra = getattr(report, 'extra', [])
    if report.when == 'call' or report.when == "setup":
        xfail = hasattr(report, 'wasxfail')
        if (report.skipped and xfail) or (report.failed and not xfail):
            file_name = report.nodeid.replace("::", "_")+".png"
            screen_img = _capture_screenshot()
            if file_name:
                html = '<div><img src="data:image/png;base64,%s" alt="screenshot" style="width:600px;height:300px;" ' \
                       'onclick="window.open(this.src)" align="right"/></div>' % screen_img
                extra.append(pytest_html.extras.html(html))
        report.extra = extra

def _capture_screenshot():
    '''
    截图保存为base64，展示到html中
    :return:
    '''
    if b:
        return b.get_screenshot_as_base64()
    else:
        return '未截图'

@pytest.fixture(scope="session", autouse=True)
def browser():
    global b
    if b is None:
        '''
            chrome的一些设置，有助测试，注释的部分自行选择
        '''
        chrome_options = Options()
        # 去除"Chrome正在受到自动化测试软件的控制"弹出框信息
        chrome_options.add_experimental_option('useAutomationExtension', False)
        chrome_options.add_experimental_option('excludeSwitches', ['--enable-automation'])
        # 启动最大化
        chrome_options.add_argument('--start-maximized')
        # 规避不必要的BUG出现
        chrome_options.add_argument('--disable-gpu')
        chrome_options.add_argument('-ignore-certificate-errors')
        chrome_options.add_argument('-ignore -ssl-errors')
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        prefs = {}
        prefs["credentials_enable_service"] = False
        prefs["profile.password_manager_enabled"] = False
        chrome_options.add_experimental_option("prefs", prefs)
        # 无头（无界面）模式
        # chrome_options.add_argument('--headless')
        # 设置手机浏览器头（模拟手机端web）
        # chrome_options.add_argument('User-Agent=Mozilla/5.0 (Linux; U; Android 8.1.0; zh-cn; BLA-AL00 Build/HUAWEIBLA-AL00) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/57.0.2987.132 MQQBrowser/8.9 Mobile Safari/537.36')
        # 禁止图片
        # chrome_options.add_argument('blink-settings=imagesEnabled=false')
        # 指定chrome.exe路径
        # chrome_options.binary_location = '*\\chrome.exe'
        try:
            b = webdriver.Chrome(chrome_options=chrome_options)
        except Exception as e:
            chromedriver_download()
            b = webdriver.Chrome(chrome_options=chrome_options)
        except TimeoutError as e:
            print('自己手动下载吧，https://npm.taobao.org/mirrors/chromedriver/')
    b.implicitly_wait(PUBLIC_VARS['WebDriverWait'])
    yield b
    b.quit()



#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'GOL_VARS.py'
__create_time__ = '2020/7/15 23:15'

PUBLIC_VARS = {
    'WebDriverWait': 30,
    'poll_frequency': 0.5,
    'page_wait_timeout': 10
}


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Proxy.py'
__create_time__ = '2021/1/12 23:22'
'''
    代理服务器的查看、启动、关闭
'''
from urllib import request, error
from argparse import ArgumentParser
from typing import Union, Optional
from xiaobaiauto2.utils import win_reg_write, win_reg_read
from xiaobaiauto2.__version__ import __version__

class ProxyServer(object):
    proxy: Optional[str] = '127.0.0.1:8080'  # default use mitmproxy
    disProxy: Optional[str] = 'localhost;127.*;192.*;10.*'
    def StartProxy(self):
        ''' 启动系统代理服务器 '''

    def StopProxy(self):
        ''' 关闭系统代理服务器 '''

    def StatusProxy(self):
        ''' 查看系统代理服务器状态 '''

class WindowsProxy(ProxyServer):
    _path = 'Software\Microsoft\Windows\CurrentVersion\Internet Settings'
    _name1 = 'ProxyEnable'
    _name2 = 'ProxyServer'
    _name3 = 'ProxyOverride'
    def _setProxy(self, enable=0, proxy=None, ignoreip=None):
        win_reg_write(self._path, self._name1, enable)
        win_reg_write(self._path, self._name2, proxy)
        win_reg_write(self._path, self._name3, ignoreip)

    def getProxy(self, path, name):
        return win_reg_read(path, name)

    def StartProxy(self):
        try:
            self._setProxy(1, self.proxy, self.disProxy)
            print('启动代理已设置为:', self.proxy)
        except Exception as e:
            print('启动代理失败', e)

    def StopProxy(self):
        try:
            self._setProxy(0, '', '')
            print('代理已关闭成功')
        except Exception as e:
            print('关闭代理失败', e)

    def StatusProxy(self):
        try:
            if self.getProxy(self._path, self._name1)[0] == 1:
                print('代理已启动')
            else:
                print('代理已关闭')
        except Exception as e:
            print('查询代理状态失败', e)

    def LiveProxy(self):
        try:
            f = self.getProxy(self._path, self._name1)[0]
        except Exception as e:
            print(e)
            exit(0)
        if f == 1:
            try:
                response = request.urlopen(url='https://www.baidu.com', timeout=5)
                if bytes('全球最大的中文搜索引擎', 'utf-8') in response.read():
                    print('当前代理有效')
                else:
                    print('当前代理无效')
            except error.URLError as e:
                print('当前代理无效', e)
        else:
            print('未使用代理')


class unixProxy(ProxyServer):
    ''' 暂不支持， 自行配置/etc/profile '''

def cmd():
    arg = ArgumentParser(prog='xiaobaiauto2Proxy',
                         description=f'小白科技·设置系统代理·{__version__}')
    arg.add_argument('-c', '--checkproxy', type=int, default=0, choices=(0, 1),
                     help='校验代理ip是否可用，(默认)0不校验，1校验')
    arg.add_argument('-d', '--disproxy', type=str, default='localhost;127.*;192.*;10.*',
                     help='排除规则，样例：localhost;127.*;192.*;10.*')
    arg.add_argument('-e', '--enable', type=int, default=2, choices=(0, 1, 2),
                     help='0关闭代理服务器, 1启动代理服务器, (默认)2查看代理服务器状态')
    arg.add_argument('-p', '--proxy', type=str, default='127.0.0.1:8080', help='代理IP规则，样例：127.0.0.1:8080')
    par = arg.parse_args()
    wproxy = WindowsProxy()
    wproxy.proxy = par.proxy
    wproxy.disProxy = par.disproxy
    if par.checkproxy == 0:
        if par.enable == 0:
            wproxy.StopProxy()
        elif par.enable == 1:
            wproxy.StartProxy()
        else:
            wproxy.StatusProxy()
    else:
        wproxy.LiveProxy()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'help_ui.py'
__create_time__ = '2020/7/22 21:40'

from tkinter import filedialog, messagebox
from tkinter import *
from tkinter import ttk
from os.path import splitext
from yaml import full_load, FullLoader
import json
from sqlite3 import connect
from xiaobaiauto2.utils.xiaobaiauto2db import DB
from xiaobaiauto2.config.config import CASE_TYPE

_case_config = CASE_TYPE()
filetypes = _case_config.filepytes

class HELP_UI:
    def read_case_file(self):
        fr = filedialog.askopenfile(filetypes=filetypes)
        try:
            if fr is not None and splitext(fr.name)[1] in ['.txt', '.csv']:
                all_data = fr.buffer.readlines()
                for c in all_data:
                    _c = c.decode('utf-8').split(_case_config.caseSeparator)
                    self.newrow(_c[0], _c[1], _c[2], _c[3], _c[4])
            elif fr is not None and splitext(fr.name)[1] in ['.json', '.yaml']:
                all_data = fr.buffer.read().decode('utf-8')
                if splitext(fr.name)[1] == '.json':
                    case_data = list(json.loads(all_data).get('cases'))
                else:
                    case_data = full_load(open(fr.name, 'r', encoding='utf-8')).get('cases')
                for c in case_data:
                    try:
                        self.newrow(c[0], c[1], c[2], c[3], c[4])
                    except IndexError as e:
                        messagebox.showerror('错误', e)
        except UnicodeError as e:
            messagebox.showerror('错误', e)

    def save_case_file(self):
        if messagebox.askyesno('温馨提示：', '您是否保存数据？若不保存只导出模板文件'):
            fw = filedialog.asksaveasfile(filetypes=filetypes)
            if fw is not None:
                fwdata = open(fw.name, 'w', encoding='utf-8')
                if splitext(fw.name)[1] in ['.txt', '.csv']:
                    for i in range(self.cmd.__len__()):
                        fwdata.write("\r\n" + str(self.cmd[i]).replace('\r\n', '').replace('\n', '') + _case_config.caseSeparator +
                                     str(self.loc[i]).replace('\r\n', '').replace('\n', '') + _case_config.caseSeparator +
                                     str(self.data[i]).replace('\r\n', '').replace('\n', '') + _case_config.caseSeparator +
                                     str(self.contains_assert[i]).replace('\r\n', '').replace('\n', '') + _case_config.caseSeparator +
                                     str(self.equal_assert[i]).replace('\r\n', '').replace('\n', '') + _case_config.caseSeparator)
                elif splitext(fw.name)[1] in ['.yaml']:
                    _header = '---\r\ncases:\r\n'
                    _body = ''
                    _foot = '\n}'
                    for i in range(self.cmd.__len__()):
                        _body += f'''\r\n\t\t[
                            \r\t\t\t"%s"
                        ]''' % str(self.cmd[i]).replace(r'\r\n', '').replace(r'\n', '')
                        fwdata.write(_header + _body + _foot)
                fwdata.close()
        else:
            filedialog.asksaveasfilename(filetypes=filetypes)

    def treeview_sort_column(self, tv, col, reverse):  # Treeview、列名、排列方式
        l = [(tv.set(k, col), k) for k in tv.get_children('')]
        l.sort(reverse=reverse)  # 排序方式
        # rearrange items in sorted positions
        for index, (val, k) in enumerate(l):  # 根据排序后索引移动
            tv.move(k, '', index)
        tv.heading(col, command=lambda: self.treeview_sort_column(tv, col, not reverse))  # 重写标题，使之成为再点倒序的标题

    def set_cell_value(self, event):  # 双击进入编辑状态
        for item in self.data_treeview.selection():
            item_text = self.data_treeview.item(item, "values")
        column = self.data_treeview.identify_column(event.x)  # 列
        row = self.data_treeview.identify_row(event.y)  # 行
        cn = int(str(column).replace('#', ''))
        if row != '':
            rn = int(str(row).replace('I', ''))
            entryedit = Text(self.f_edit, width=int(self.columns_width[cn-1]/10), height=1)#  + (cn - 1) * 16, height=1)
            entryedit.place(x=16 + (cn - 1) * self.columns_width[cn-1], y=6 + rn * 20)
            treeview = self.data_treeview
            def saveedit():
                if entryedit.get(0.0, "end") != '\n':
                    treeview.set(item, column=column, value=entryedit.get(0.0, "end"))
                entryedit.destroy()
                okb.destroy()
            okb = ttk.Button(self.f_edit, text='OK', width=4, command=saveedit)
            okb.place(x=90 + (cn - 1) * self.columns_width[cn-1], y=2 + rn * 20)
        else:
            messagebox.showerror('错误', '还没有数据别点了！')

    def newrow(self, cmd='指令', loc='定位', data='数据', ccontains_assert='包含断言', equal_assert='相等断言'):
        self.cmd.append(cmd)
        self.loc.append(loc)
        self.data.append(data)
        self.contains_assert.append(ccontains_assert)
        self.equal_assert.append(equal_assert)
        self.data_treeview.insert('',
                                  len(self.cmd) - 1,
                                  values=(
                                      self.cmd[len(self.cmd) - 1],
                                      self.loc[len(self.loc) - 1],
                                      self.data[len(self.data) - 1],
                                      self.contains_assert[len(self.contains_assert) - 1],
                                      self.equal_assert[len(self.equal_assert) - 1],
                                  ))
        self.data_treeview.update()
        self.newb.place(x=120, y=(len(self.cmd) - 1) * 20 + 45)
        self.newb.update()

    def edit_case_ui(self, master):
        f_body = Frame(master=master, width=600)
        f_tool = Frame(master=f_body, )
        self.f_edit = Frame(master=f_body, width=600)
        Button(master=f_tool, text='导入用例', command=self.read_case_file).pack(side=LEFT, fill=Y)
        Button(master=f_tool, text='导出用例', command=self.save_case_file).pack(side=RIGHT, fill=Y)
        f_tool.pack(side=TOP, fill=X)
        # 添加 tableview
        # columns = ("CMD", "LOC", "DATA", "CONTAINS_ASSERT", "EQUAL_ASSERT")
        columns = ("指令", "定位", "数据", "包含断言", "相等断言")
        self.columns_width = [100, 200, 150, 150, 150]
        self.cmd = []
        self.loc = []
        self.data = []
        self.contains_assert = []
        self.equal_assert = []
        self.data_treeview = ttk.Treeview(self.f_edit, height=26, show="headings", columns=columns)  # 表格
        self.data_treeview.column("指令", width=self.columns_width[0], anchor='center')  # 表示列,不显示
        self.data_treeview.column("定位", width=self.columns_width[1], anchor='center')  # 表示列,不显示
        self.data_treeview.column("数据", width=self.columns_width[2], anchor='center')  # 表示列,不显示
        self.data_treeview.column("包含断言", width=self.columns_width[3], anchor='center')  # 表示列,不显示
        self.data_treeview.column("相等断言", width=self.columns_width[4], anchor='center')  # 表示列,不显示
        self.data_treeview.heading("指令", text="指令")  # 显示表头
        self.data_treeview.heading("定位", text="定位")  # 显示表头
        self.data_treeview.heading("数据", text="数据")  # 显示表头
        self.data_treeview.heading("包含断言", text="包含断言")  # 显示表头
        self.data_treeview.heading("相等断言", text="相等断言")  # 显示表头
        self.data_treeview.pack(side=LEFT, fill=BOTH)
        self.data_treeview.bind('<Double-1>', self.set_cell_value)  # 双击左键进入编辑
        self.newb = ttk.Button(self.f_edit, text='新建用例', width=20, command=self.newrow)
        self.newb.place(x=120, y=(len(self.cmd)) * 20 + 45)
        for col in columns:  # 绑定函数，使表头可排序
            self.data_treeview.heading(
                col, text=col, command=lambda _col=col: self.treeview_sort_column(self.data_treeview, _col, False))
        self.f_edit.pack()# side=BOTTOM, fill=X)
        f_body.pack()

    def vent_ui(self, master):
        '''环境检查'''
        pass

    def db_view(self, master):
        '''数据配置'''
        db = DB()
        db.select('select * from keyword')
        f_body = Frame(master=master, width=600)
        f_tool = Frame(master=f_body, )
        self.f_edit = Frame(master=f_body, width=600)
        Button(master=f_tool, text='导入关键词', command=self.read_case_file).pack(side=LEFT, fill=Y)
        Button(master=f_tool, text='导出关键词', command=self.save_case_file).pack(side=RIGHT, fill=Y)
        f_tool.pack(side=TOP, fill=X)
        # 添加 tableview
        # columns = ("CMD", "LOC", "DATA", "CONTAINS_ASSERT", "EQUAL_ASSERT")
        columns = ("编号", "命令", "关键词", "测试类型", "元素命令", "设备命令", "代码", "创建时间", "是否删除")
        self.columns_width = [100, 200, 150, 150, 150]
        self.cmd = []
        self.loc = []
        self.data = []
        self.contains_assert = []
        self.equal_assert = []
        self.data_treeview = ttk.Treeview(self.f_edit, height=26, show="headings", columns=columns)  # 表格
        self.data_treeview.column("指令", width=self.columns_width[0], anchor='center')  # 表示列,不显示
        self.data_treeview.column("定位", width=self.columns_width[1], anchor='center')  # 表示列,不显示
        self.data_treeview.column("数据", width=self.columns_width[2], anchor='center')  # 表示列,不显示
        self.data_treeview.column("包含断言", width=self.columns_width[3], anchor='center')  # 表示列,不显示
        self.data_treeview.column("相等断言", width=self.columns_width[4], anchor='center')  # 表示列,不显示
        self.data_treeview.heading("指令", text="指令")  # 显示表头
        self.data_treeview.heading("定位", text="定位")  # 显示表头
        self.data_treeview.heading("数据", text="数据")  # 显示表头
        self.data_treeview.heading("包含断言", text="包含断言")  # 显示表头
        self.data_treeview.heading("相等断言", text="相等断言")  # 显示表头
        self.data_treeview.pack(side=LEFT, fill=BOTH)
        self.data_treeview.bind('<Double-1>', self.set_cell_value)  # 双击左键进入编辑
        self.newb = ttk.Button(self.f_edit, text='新建用例', width=20, command=self.newrow)
        self.newb.place(x=120, y=(len(self.cmd)) * 20 + 45)
        for col in columns:  # 绑定函数，使表头可排序
            self.data_treeview.heading(
                col, text=col, command=lambda _col=col: self.treeview_sort_column(self.data_treeview, _col, False))
        self.f_edit.pack()  # side=BOTTOM, fill=X)
        f_body.pack()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test01.py'
__create_time__ = '2020/7/17 2:15'

import pytest

def setup_module():
    print("整个文档执行前执行一次")

def teardown_module():
    print("整个文件执行后执行一次")

def setup_function():
    print("执行每个方法前执行一次，不适合单接口")

def teardown_function():
    print("执行每个方法后执行一次，不适合单接口")


def test_xiaobai_api():
    print("执行接口测试")
    assert '实际返回值' != '预期值'

class TestClass(object):

    def setup_class(self):
        print("setup_class(self)：每个类之前执行一次")

    def teardown_class(self):
        print("teardown_class(self)：每个类之后执行一次")

    def test_xiaobai_api2(self):
        print("执行接口测试")
        assert '实际返回值' != '预期值'

# if __name__=="__main__":
#     pytest.main(["-s", "test_xiaobai_api.py"])
# 或者命令行运行生成HTML报告，命令如下
# pytest --html=report.html --self-contained-html


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test_WebUI.py'
__create_time__ = '2020/7/15 23:34'

from xiaobaiauto2.xiaobaiauto2 import web_action, cmd
from xiaobaiauto2.utils.xiaobaiauto2Email import send_email
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS
from xiaobaiauto2.config.config import EMAILCONFIG
from os import path
from time import sleep
import pytest

@pytest.mark.xiaobai_web
def test_Case1(browser):
    web_action(browser, cmd=cmd.打开网页, loc='', data='http://www.baidu.com')
    web_action(browser, cmd=cmd.输入, loc='//*[@id="kw"]', data='小白科技')
    web_action(browser, cmd=cmd.点击, loc='//*[@id="su"]')
    web_action(browser, cmd=cmd.wait, data=3)
    web_action(browser, cmd=cmd.标题, contains_assert='小白')

@pytest.mark.xiaobai_web
def test_Case2(browser):
    web_action(browser, '打开', loc='', data='https://www.baidu.com')
    web_action(browser, '输入', '//*[@id="kw"]', '漫宅')
    web_action(browser, '点击', '//*[@id="su"]')
    web_action(browser, '停止时间', data=3)
    web_action(browser, '标题', contains_assert='漫')
    web_action(browser, '关闭')

@pytest.mark.last
def test_last():
    ''' 本次结束测试结束，发送邮件 '''
    print('测试结束了，发个邮件吧')
    sleep(2)
    _emil = EMAILCONFIG()
    _cur_path = path.abspath(path.curdir)
    PUBLIC_VARS['report'] = 'report.html'  # 命令行您期望的报告文件名
    if 'report' in PUBLIC_VARS.keys() and '' != PUBLIC_VARS['report']:
        if path.isfile(_cur_path + '/' + PUBLIC_VARS['report']):
            send_email(_cur_path + '/' + PUBLIC_VARS['report'])
    elif '' != _emil.report:
        if path.isfile(_cur_path + '/' + _emil.report):
            send_email(_cur_path + '/' + _emil.report)

@pytest.mark.run(order=1)
def test_first():
    print('测试开始了，准备邮件信息')
    emil = {
        'sender': 'tser@163.com',
        'receiver': ['807447312@qq.com', 'qiankuny@163.com', '912194099@qq.com'],
        'smtpserver': 'smtp.163.com',
        'smtp_port': 25,
        'username': 'tser@xiaobaiit.com',
        'password': '',
        'subject': '小白自动化测试报告',
        'report': 'report.html'
    }
    PUBLIC_VARS.update(emil)

# if __name__ == '__main__':
#     pytest.main(['test_WebUI.py', '--html=report.html', ' -s'])


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2.py'
__create_time__ = '2020/7/2 18:20'

from xiaobaiauto2.utils.xiaobaiauto2db import DB
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS
from requests import request
from jmespath import search
from re import findall
from typing import Optional
from ddddocr import DdddOcr
from os import path, remove
from selenium.webdriver.remote.webdriver import WebDriver, WebElement
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from wsgiref.simple_server import make_server

allapi = dict()

def get(key: str = ''):
    if key is not '':
        return PUBLIC_VARS.get(key)

def set(key: str = '', value=None):
    if key is not '':
        PUBLIC_VARS[key] = value

def app(environ, start_response):
    if environ.get('PATH_INFO') in allapi:
        status = '200 OK'
        headers = [('Content-type', 'text/plain; charset=utf-8'),
                   ('project', 'xiaobaiauto2'),
                   ('Server', 'xiaobaikeji')]
        start_response(status, headers)
        return [allapi.get(environ.get('PATH_INFO'))]
    else:
        status = '404 Not Found'
        headers = [('Content-type', 'text/plain; charset=utf-8'),
                   ('project', 'xiaobaiauto2'),
                   ('Server', 'xiaobaikeji')]
        start_response(status, headers)
        return [b'Not Found']

def MockServer(**kwargs):
    '''
    其他参数待扩展
    :param kwargs:
    :return:
    '''
    if 'uri' in kwargs.keys():
        uri = kwargs.get('uri')
    else:
        uri = '/'
    if 'response' in kwargs.keys():
        response = str(kwargs.get('response')).encode('utf-8')
    else:
        response = '{"errcode": 200, "msg": "请求成功", "data": {}}'.encode('utf-8')
    global allapi
    if uri not in allapi.keys():
        allapi[uri] = response
    def xiaobai(func):
        def core(**kwargs):
            if 'host' in kwargs.keys():
                host = kwargs.get('host')
            else:
                host = '0.0.0.0'
            if 'port' in kwargs.keys():
                port = kwargs.get('port')
            else:
                port = 6666
            with make_server(host=host, port=port, app=app) as httpd:
                print(f"xiaobaiMock Serving on port {port}...")
                try:
                    httpd.serve_forever()
                except KeyboardInterrupt:
                    print("[Ctrl+C] Shutting down.")
                    httpd.server_close()
        return core
    return xiaobai

mock = MockServer

def verify_check(image_element: WebElement=None, input_element:WebElement =None):
    '''
    验证码识别过程:
    1、通过验证码图片元素进行截图并保存为图片
    2、通过OCR识别并获取结果
    3、将结果填写到验证码输入框
    :param image_element: 验证码图片元素
    :param input_element: 验证码输入框元素
    :return:
    '''
    IMAGE_FILE_NAME = 'verify.png'
    image_element.screenshot(IMAGE_FILE_NAME)
    if path.isfile(IMAGE_FILE_NAME):
        ocr = DdddOcr(old=True)
        with open(IMAGE_FILE_NAME, 'rb') as f:
            image = f.read()
        res = ocr.classification(image)
        if '' != res:
            input_element.send_keys(res)
        else:
            ''' 识别失败 '''
        remove(IMAGE_FILE_NAME)

def cmd2action(b=None, r=None, loc=None, data=None, contains_assert=None, equal_assert=None):
    wait = WebDriverWait(b, PUBLIC_VARS['WebDriverWait'], PUBLIC_VARS['poll_frequency'])
    if r[0][0] == 1:
        if loc not in ('', None):
            if isinstance(b, WebDriver):
                try:
                    e = wait.until(EC.presence_of_element_located((By.XPATH, loc)))
                except TimeoutError as e:
                    try:
                        e = wait.until(lambda b: b.find_element(by=By.XPATH, value=loc))
                    except TimeoutError as e:
                        e = b.find_element(by=By.XPATH, value=loc)
                if '%s' in r[0][2] or '%d' in r[0][2]:
                    eval(r[0][2] % data)
                    if contains_assert:
                        assert contains_assert in eval(r[0][2] % data)
                    elif equal_assert:
                        assert equal_assert == eval(r[0][2] % data)
                else:
                    eval(r[0][2])
                    if contains_assert:
                        assert contains_assert in eval(r[0][2])
                    elif equal_assert:
                        assert equal_assert == eval(r[0][2])
    elif r[0][1] == 1:
        if isinstance(b, WebDriver):
            if '%s' in r[0][2] or '%d' in r[0][2]:
                eval(r[0][2] % data)
                if contains_assert:
                    assert contains_assert in eval(r[0][2] % data)
                elif equal_assert:
                    assert equal_assert == eval(r[0][2] % data)
            else:
                eval(r[0][2])
                if contains_assert:
                    assert contains_assert in eval(r[0][2])
                elif equal_assert:
                    assert equal_assert == eval(r[0][2])
    else:
        if '%s' in r[0][2] or '%d' in r[0][2]:
            eval(r[0][2] % data)
            if contains_assert:
                assert contains_assert in eval(r[0][2] % data)
            elif equal_assert:
                assert equal_assert == eval(r[0][2] % data)
        else:
            eval(r[0][2])
            if contains_assert:
                assert contains_assert in eval(r[0][2])
            elif equal_assert:
                assert equal_assert == eval(r[0][2])

def action(app_type=1, b=None, cmd=None, loc=None, data=None, contains_assert=None, equal_assert=None):
    '''
    操作方法
    :param app_type: 操作对象类型1为web，2为app
    :param b: 操作对象
    :param cmd: 操作命令
    :param loc: 操作路径
    :param data: 操作数据
    :param contains_assert: 包含校验
    :param equal_assert: 相等校验
    :return:
    '''
    if b is not None:
        db = DB()
        r = db.select(f"select is_element,is_driver,code\
                      from keyword\
                      where testtype like '%{app_type}%' and command like '%{cmd}%' or key like '%{cmd}%' limit 1;")
        cmd2action(b, r, loc, data, contains_assert, equal_assert)

def web_action(b=None, cmd=None, loc=None, data=None, contains_assert=None, equal_assert=None):
    action(app_type=1, b=b, cmd=cmd, loc=loc, data=data, contains_assert=contains_assert, equal_assert=equal_assert)

def app_aciton(b=None, cmd=None, loc=None, data=None, contains_assert=None, equal_assert=None):
    action(app_type=2, b=b, cmd=cmd, loc=loc, data=data, contains_assert=contains_assert, equal_assert=equal_assert)

class cmd(object):
    ''' UI动作 '''
    openurl = '打开网页'
    click = '点击'
    sendkey = '输入'
    refresh = '刷新'
    back = '后退'
    close = '关闭'
    quit = '退出'
    tag = '标签'
    attr = '属性'
    curl = 'URL'
    title = '标题'
    frame = '内嵌页'
    window = '标签页[序号(1开始)]'
    alter0 = 'JS_确定'
    alter1 = 'JS_取消'
    alter2 = 'JS_输入框'
    alter3 = 'JS_文本'
    wait = '停止'
    script = '脚本'
    addcookie = '添加cookie'
    swipe = '滑屏'
    dscreenshot = '截屏'
    escreenshot = '元素截图'
    findtext = '识别验证码'
    location = '坐标'
    page_source = '网页源码'
    打开网页 = 'OPENURL'
    点击 = 'CLICK'
    输入 = 'SENDKEY'
    刷新 = 'REFRESH'
    后退 = 'BACK'
    关闭 = 'CLOSE'
    退出 = 'QUIT'
    标签 = 'TAG'
    属性 = 'ATTR'
    URL = 'CURL'
    标题 = 'TITLE'
    内嵌页 = 'FRAME'
    标签页 = 'WINDOW'
    JS_确定 = 'ALERT0'
    JS_取消 = 'ALERT1'
    JS_输入框 = 'ALERT2'
    JS_文本 = 'ALERT3'
    停止 = 'WAIT'
    脚本 = 'SCRIPT'
    添加cookie = 'ADDCOOKIE'
    滑屏 = 'SWIPE'
    截屏 = 'DSCREENSHOT'
    元素截图 = 'ESCREENSHOT'
    识别验证码 = 'FINDTEXT'
    坐标 = 'LOCATION'
    网页源码 = 'page_source'

class assert_mode(object):
    ''' 断言格式 '''
    json = 'json'
    contains = 'contains'

class extract_at(object):
    ''' 提取位置 '''
    status = 'status'
    headers = 'headers'
    body = 'body'
    all = 'all'

class extract_mode(object):
    ''' 提取方式 '''
    json = 'json'
    re = 're'

def _str_format(params):
    '''
    字符串格式化，将参数化数据替换为预设值
    :param params:  待处理对象
    :return:        处理后对象
    '''
    if isinstance(params, dict):
        ''' dict格式 '''
        for k, v in params.items():
            params[k] = _str_format(v)
        return params

    elif isinstance(params, list):
        ''' list格式 '''
        for i, v in enumerate(params):
            params[i] = _str_format(v)
        return params

    elif isinstance(params, tuple):
        ''' tuple格式 '''
        params = list(params)
        for i, v in enumerate(params):
            params[i] = _str_format(v)
        return tuple(params)

    elif isinstance(params, set):
        ''' set格式 '''
        params = list(params)
        for i, v in enumerate(params):
            params[i] = _str_format(v)
        return set(params)

    elif isinstance(params, bytes):
        ''' byte格式 '''
        return bytes(_str_format(str(params, 'utf-8')), 'utf-8')

    elif isinstance(params, str):
        ''' 字符串格式 '''
        keys = findall('{(.+?)}', params)
        if len(keys) != 0:
            for key in keys:
                source = '{' + key + '}'
                target = PUBLIC_VARS.get(key)
                if target:
                    params = params.replace(source, target.__str__())
        return params
    else:
        ''' 其他格式 '''
        return params

def apiTestCase(func):
    '''
    接口描述的装饰器
    :param func:
    :return:
    '''
    def xiaobai(**kwargs):
        kwargs = _str_format(kwargs)
        d = func.__name__.split('_')
        try:
            data_list = d[:d.index('')]
        except ValueError as e:
            data_list = d
        data_list = _str_format(data_list)
        doc = func.__doc__
        if PUBLIC_VARS.get('base') in ['', None]:
            PUBLIC_VARS['base'] = 'https://bbs.jicaiyunshang.com/'
        kw = kwargs.copy()
        keyword = ['assert_mode', 'assert_path', 'assert_value', 'extract_at', 'extract_mode', 'extract_path',
                   'extract_name']
        for k in keyword:
            if k in kw.keys():
                kw.pop(k)
        PUBLIC_VARS['base'] = _str_format(PUBLIC_VARS.get('base'))
        res = request(method=data_list[0], url=PUBLIC_VARS.get('base') + '/'.join(data_list[1:]), **kw)
        res.encoding = 'utf-8'
        if 'json' == kwargs.get('assert_mode') and \
                'assert_mode' in kwargs.keys() and 'application/json' in res.headers.get('Content-Type'):
            assert kwargs.get('assert_value') == search(kwargs.get('assert_path'), res.json())
        elif 'contains' == kwargs.get('assert_mode') and 'assert_mode' in kwargs.keys():
            assert kwargs.get('assert_value') in res.text
        if 'status' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            PUBLIC_VARS[kwargs.get('extract_name')] = res.status_code
        elif 'headers' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            if 'json' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                value = search(kwargs.get('extract_path'), res.headers)
                PUBLIC_VARS[kwargs.get('extract_name')] = value if value is not None else ''
            elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                value = findall(kwargs.get('extract_path'), res.headers.__str__())
                PUBLIC_VARS[kwargs.get('extract_name')] = value if value != [] else ['']
        elif 'body' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            if 'json' == kwargs.get('extract_mode') and\
                    'extract_mode' in kwargs.keys() and 'application/json' in res.headers.get('Content-Type'):
                value = search(kwargs.get('extract_path'), res.json())
                PUBLIC_VARS[kwargs.get('extract_name')] = value if value is not None else ''
            elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                value = findall(kwargs.get('extract_path'), res.text)
                PUBLIC_VARS[kwargs.get('extract_name')] = value if value != [] else ['']
        elif 'all' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            ''' search in headers and body '''
            PUBLIC_VARS[kwargs.get('extract_name')] = []
            value = search(kwargs.get('extract_path'), res.headers)
            v = value if value is not None else ''
            PUBLIC_VARS[kwargs.get('extract_name')].append(v)
            if 'application/json' in res.headers.get('Content-Type'):
                value = search(kwargs.get('extract_path'), res.json())
                v = value if value is not None else ''
                PUBLIC_VARS[kwargs.get('extract_name')].append(v)
            else:
                value = search(kwargs.get('extract_path'), res.json())
                v = value if value != [] else ['']
                PUBLIC_VARS[kwargs.get('extract_name')].append(v)
    return xiaobai

def api_action(method: Optional[str] = 'POST', url: Optional[str] = '', **kwargs):
    '''
    接口操作方法
    :param method: 接口请求方式
    :param url: 接口地址
    :param assert_mode: 校验模式[json, contains]
    :param assert_path:  校验预期结果路径
    :param assert_value: 校验预期结果
    :param extract_at:  提取数据所在位置 [status, headers, body, all]
    :param extract_mode: 提取方式 [json, re]
    :param extract_path: 提取路径表达式
    :param extract_name: 提取数据变量名
    :param kwargs: 其他参数参考requests的api
    :return:
    例如：api_action(
            method='POST',
            url='https://bbs.jicaiyunshang.com/login',
            params='',
            data='{"username": "xiaobai", "password": "abcdef"}',
            headers={},
            assert_mode=assert_mode.json,
            assert_path='code',
            assert_value=200,
            extract_at=extract_at.body,
            extract_mode=extract_at.json,
            extract_path='data.accessToken',
            extract_name='accessToken'
        )
    '''
    kwargs = _str_format(kwargs)
    method = _str_format(method)
    url = _str_format(url)
    kw = kwargs.copy()
    keyword = ['assert_mode', 'assert_path', 'assert_value', 'extract_at', 'extract_mode', 'extract_path',
               'extract_name']
    for k in keyword:
        if k in kw.keys():
            kw.pop(k)
    res = request(method=method.upper(), url=url, **kw)
    res.encoding = 'utf-8'
    if 'json' == kwargs.get('assert_mode') and \
            'assert_mode' in kwargs.keys() and 'application/json' in res.headers.get('Content-Type'):
        assert kwargs.get('assert_value') == search(kwargs.get('assert_path'), res.json())
    elif 'contains' == kwargs.get('assert_mode') and 'assert_mode' in kwargs.keys():
        assert kwargs.get('assert_value') in res.text
    if 'status' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
        PUBLIC_VARS[kwargs.get('extract_name')] = res.status_code
    elif 'headers' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
        if 'json' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
            value = search(kwargs.get('extract_path'), res.headers)
            PUBLIC_VARS[kwargs.get('extract_name')] = value if value is not None else ''
        elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
            value = findall(kwargs.get('extract_path'), res.headers.__str__())
            PUBLIC_VARS[kwargs.get('extract_name')] = value if value != [] else ['']
    elif 'body' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
        if 'json' == kwargs.get('extract_mode') and \
                'extract_mode' in kwargs.keys() and 'application/json' in res.headers.get('Content-Type'):
            value = search(kwargs.get('extract_path'), res.json())
            PUBLIC_VARS[kwargs.get('extract_name')] = value if value is not None else ''
        elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
            value = findall(kwargs.get('extract_path'), res.text)
            PUBLIC_VARS[kwargs.get('extract_name')] = value if value != [] else ['']
    elif 'all' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
        ''' search in headers and body '''
        PUBLIC_VARS[kwargs.get('extract_name')] = []
        value = search(kwargs.get('extract_path'), res.headers)
        v = value if value is not None else ''
        PUBLIC_VARS[kwargs.get('extract_name')].append(v)
        if 'application/json' in res.headers.get('Content-Type'):
            value = search(kwargs.get('extract_path'), res.json())
            v = value if value is not None else ''
            PUBLIC_VARS[kwargs.get('extract_name')].append(v)
        else:
            value = search(kwargs.get('extract_path'), res.json())
            v = value if value != [] else ['']
            PUBLIC_VARS[kwargs.get('extract_name')].append(v)
    return res


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'client.py'
__create_time__ = '2020/9/18 13:50'

from xiaobaiauto2.xiaobaiauto2 import *
from typing import Optional
from requests import request, Response
from jmespath import search
from re import findall
import pysnooper

env: Optional[dict] = {}   # 环境变量

def apiCase(func):
    def inner(**kwargs):
        data = func.__name__.replace('test_', '').replace('_test', '')
        data_list = data.split('_')
        doc = func.__doc__
        global env
        if env.get('base') in ['', None]:
            env['base'] = 'https://www.jicaiyunshang.com/'
        kw = kwargs.copy()
        keyword = ['assert_mode','assert_path','assert_value','extract_at','extract_mode','extract_path','extract_name']
        for k in keyword:
            if k in kw.keys():
                kw.pop(k)
        res = request(method=data_list[0], url=env.get('base') + '/'.join(data_list[1:]), **kw)

        if 'json' == kwargs.get('assert_mode') and\
                'assert_mode' in kwargs.keys() and\
                'application/json' in res.headers.get('content-type'):
            assert kwargs.get('assert_value') == search(kwargs.get('assert_path'), res.json())
        elif 'contains' == kwargs.get('assert_mode') and 'assert_mode' in kwargs.keys():
            assert kwargs.get('assert_value') in res.text
        if 'status' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            env[kwargs.get('extract_name')] = res.status_code
        elif 'headers' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            if 'json' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                env[kwargs.get('extract_name')] = search(kwargs.get('extract_path'), res.headers)
            elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                env[kwargs.get('extract_name')] = findall(kwargs.get('extract_path'), res.headers.__str__())
        elif 'body' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            if 'json' == kwargs.get('extract_mode') and\
                    'extract_mode' in kwargs.keys() and \
                    'application/json' in res.headers.get('content-type'):
                env[kwargs.get('extract_name')] = search(kwargs.get('extract_path'), res.json())
            elif 're' == kwargs.get('extract_mode') and 'extract_mode' in kwargs.keys():
                env[kwargs.get('extract_name')] = findall(kwargs.get('extract_path'), res.text)
        elif 'all' == kwargs.get('extract_at') and 'extract_at' in kwargs.keys():
            ''' search in headers and body '''
            env[kwargs.get('extract_name')] = []
            env[kwargs.get('extract_name')].append(search(kwargs.get('extract_path'), res.headers))
            if 'application/json' in res.headers.get('content-type'):
                env[kwargs.get('extract_name')].append(search(kwargs.get('extract_path'), res.json()))
            else:
                env[kwargs.get('extract_name')].append(findall(kwargs.get('extract_path'), res.text))
    return inner

class mode(object):
    ''' 断言格式 '''
    json = 'json'
    re = 're'
    contains = 'contains'

class at(object):
    ''' 提取位置 '''
    status = 'status'
    headers = 'headers'
    body = 'body'
    all = 'all'

class apiTestCase(object):
    res = Response()
    @property
    def res_text(self):
        ''' Text '''
        self.res.encoding = 'utf-8'
        return self.res.text

    @property
    def res_content(self):
        ''' content '''
        return self.res.content

    @property
    def res_json(self):
        ''' json '''
        return self.res.json()

    def assertion(self, at: Optional[str] = 'body',
                  mode: Optional[str] = 'json',
                  path: Optional[str] = '',
                  value=None):
        ''' assert '''

    def paramsHandles(self, params):
        '''
            username={name0}&password={pwd0}
            name0 and pwd0 find by PUBLIC_VARS

        '''

    def __call__(self, func):
        def run(**kwargs):
            '''
            1、参数化
            2、请求
            3、断言、提取
            公共参数 params, data, headers, at, mode, path, value等
            '''
            if kwargs.get('at') == 'body':
                if 'json' in self.res.headers.get('content-type'):
                    content = self.res_json
                else:
                    content = self.res_text
            if 'assert' in func.__name__.lower():
                ''' 执行断言的处理 '''
                data = self.res
            elif 'extract' in func.__name__.lower():
                ''' 执行提取的处理 '''
        return run


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'cases.py'
__create_time__ = '2020/11/25 16:01'

from xiaobaiauto2.xiaobaiauto2 import *

'''
    ······业务层接口······
    方法名定义规则：
        接口方式_接口路径(路径中/替换为_)
    例如：
        POST  /api/user/login --> post_api_user_login
        或
        POST  /api/user/login --> post_api_user_login__其它描述信息
    注意：方法参数与方法体不需要写任何内容，可以写描述信息（如下）
'''

@apiTestCase
def post_api_user_login():
    ''' 用户登录接口 '''

@apiTestCase
def post_api_getList():
    ''' 获取列表接口 '''

@apiTestCase
def post_api_getInfo():
    ''' 获取详情接口 '''

@apiTestCase
def post_api_saveOrder():
    ''' 下单接口 '''


#! /usr/bin/env python
# coding=utf-8

__auther__ = 'Tser'
__mail__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2Copy.py'
__created__ = '2022/5/20 1:03'

import pyperclip
from re import findall
from time import sleep, ctime, time
from tkinter.messagebox import askyesno, showinfo

def curl2requests(curl_str: str = None):
    '''
    解析参数文档：https://curl.se/docs/manpage.html
    解析curl中的地址、请求方法：-X、请求头：-H、请求体：--data-raw、证书不验证：--insecure，其他参数后续再追加
    '''
    if 'curl' == curl_str[:4] and '-H' in curl_str:
        code = '''# 本代码为解析代码，可能存在不确定性问题，如有问题请及时与作者微信联系：xiaobaiTser，感谢您的使用☺
                    \rfrom requests import request\
                    \rfrom re import findall\
                    \r
        '''
        curl_str = curl_str.replace('\n', '').replace('\\\r', '').replace('   ', ' ')
        url = ''
        method = 'GET'
        data = ''
        verify = True
        if '--compressed' in curl_str and 'curl -H' != curl_str[:7]:
            ''' browser '''
            URLReResult = findall('curl [\'|\"](\S+)[\'|\"]', curl_str)
            url = URLReResult[0] if URLReResult.__len__() > 0 else ''
            DataReResult = findall('--data-raw [\'|\"](\S+)[\'|\"]', curl_str)
            data = DataReResult[0] if DataReResult.__len__() > 0 else ''
            if data:
                method = 'POST'
            else:
                method = 'GET'
            if '--insecure' in curl_str:
                verify = False
        elif '--compressed' in curl_str and 'curl -H' == curl_str[:7]:
            ''' charles '''
            URLReResult = findall('--compressed [\'|\"](\S+)[\'|\"]', curl_str)
            url = URLReResult[0] if URLReResult.__len__() > 0 else ''
            DataReResult = findall('--data-binary [\'|\"](\S+)[\'|\"]', curl_str)
            data = DataReResult[0] if DataReResult.__len__() > 0 else ''
            if data:
                method = 'POST'
            else:
                method = 'GET'
            if ' -k ' in curl_str:
                verify = False
        elif '--compressed' not in curl_str and '-o' in curl_str:
            ''' fiddler '''
            if '-X' in curl_str:
                URLReResult = findall('-X [A-Z]+ [\'|\"](\S+)[\'|\"]', curl_str)
                url = URLReResult[0] if URLReResult.__len__() > 0 else ''
            else:
                URLReResult = findall('[\'|\"](.+?)[\'|\"]', curl_str)
                URLFilter = [i for i in URLReResult if ' ' not in i]
                url = URLFilter[0] if URLFilter.__len__() > 0 else ''
            methodReResult = findall('-X ([A-Z]+)', curl_str)
            method = methodReResult[0] if methodReResult.__len__() > 0 else 'GET'
        headers = {}
        header_list = findall('-H [\'|\"](.+?)[\'|\"]', curl_str)
        for value in header_list:
            header = value.split(': ')
            headers[header[0]] = header[1]
        code += f'''\
            \rurl = '{url}'\
            \rmethod = '{method}'\
            \rheaders = {headers}\
            '''
        if data:
            code += f'''\
                \rdata = '{data}'\
                '''
        if False == verify:
            code += f'''\
                \rverify = {verify}\
                '''
        if data and False == verify:
            code += '''
                \rresponse = request(method=method, url=url, headers=headers, data=data, verify=verify)\
                '''
        elif data and True == verify:
            code += '''
                \rresponse = request(method=method, url=url, headers=headers, data=data)\
                '''
        elif not data and False == verify:
            code += '''
                \rresponse = request(method=method, url=url, headers=headers, verify=verify)\
                '''
        else:
            code += '''
                \rresponse = request(method=method, url=url, headers=headers)\
                '''
        code += '''
            \r# 测试后数据判断/提取\
            \r# assert response.status_code == 200  # 判断HTTP响应状态\
            \r# var_name = response.headers()[路径]  # 提取响应头数据\
            \rif 'json' in response.headers.get('Content-Type'):\
            \r    # assert '预期结果' == response.json()[路径]  # 判断json响应体结果\
            \r    # var_name = response.json()[路径]  # 提取json响应体数据\
            \r    print(response.json())\
            \relse:\
            \r    # assert '预期结果' in response.text # 判断字符串返回值结果\
            \r    # var_name = findall('正则表达式', response.text)[0] # 正则提取数据\
            \r    print(response.text)\
            '''
        pyperclip.copy(code)
        return '已转为requests代码，请使用[Ctrl + V]粘贴使用☺'
    else:
        return '无效内容，转换失败，请微信联系作者：xiaobaiTser'

def main():
    while True:
        CPtext = pyperclip.paste()
        if 'curl ' in CPtext[:5]:
            result = askyesno(title='小白提示：', message='发现粘贴板有curl脚本是否转requests代码？')
            if 1 == result:
                ''' 代码替换 '''
                msg = curl2requests(CPtext)
                showinfo(title='小白提示：', message=msg)
        sleep(1)
        print('', end='\r')
        print(f'[{ctime()}]正在监听粘贴板中{"."*(int(time())%6)}', end='')


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'config.py'
__create_time__ = '2020/7/15 23:18'

from os import path
import logging
from xiaobaiauto2.__version__ import __version__
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS

class LOGCONFIG:
    def __init__(self):
        log_name = 'xiaobaiauto2'
        log_level = logging.DEBUG
        log_fmt = '%(asctime)s %(levelname)s %(filename)s %(funcName)s %(lineno)d: %(message)s'
        log_file_name = 'xiaobaiauto2_debug.log'
        logger = logging.getLogger(log_name)
        logger.setLevel(log_level)
        fmt = logging.Formatter(log_fmt)
        '''
        'ERROR': logging.ERROR,
        'WARNING': logging.WARNING,
        'INFO': logging.INFO,
        'DEBUG': logging.DEBUG,
        'NOTSET': logging.NOTSET
        '''
        fileHandle = logging.FileHandler(log_file_name)
        fileHandle.setLevel(logging.DEBUG)
        fileHandle.setFormatter(fmt=fmt)
        logger.addHandler(fileHandle)

class DBCONFIG:
    def __init__(self):
        if 'dbpath' in PUBLIC_VARS.keys():
            self.dbapth = PUBLIC_VARS['dbpath']
        else:
            self.dbpath = path.dirname(path.abspath(__file__)) + '/../data/xiaobaiauto2.db'

class APICONFIG:
    def __init__(self):
        if 'base' in PUBLIC_VARS.keys():
            self.base = PUBLIC_VARS['base']
        else:
            self.base = 'https://test.xiaobai.com'

class TIMERCONFIG:
    def __init__(self):
        if 'geometry' in PUBLIC_VARS.keys():
            self.geometry = PUBLIC_VARS['geometry']
        else:
            self.geometry = '500x350'
        if 'timerpath' in PUBLIC_VARS.keys():
            self.timerpath = PUBLIC_VARS['timerpath']
        else:
            self.timerpath = path.dirname(path.abspath(__file__)) + '/../data/favicon.ico'
        if 'timerTitle' in PUBLIC_VARS.keys():
            self.timerTitle = PUBLIC_VARS['timerTitle']
        else:
            self.timerTitle = '小白定时器 v' + __version__ + '  '
        if 'timerRuningTitle' in PUBLIC_VARS.keys():
            self.timerRuningTitle = PUBLIC_VARS['timerRuningTitle']
        else:
            self.timerRuningTitle = '小白定时器 v%s [运行中] ' % __version__

class EMAILCONFIG:
    def __init__(self):
        if 'sender' in PUBLIC_VARS.keys():
            self.sender = PUBLIC_VARS['sender']
        else:
            self.sender = '807447312@qq.com'
        if 'receiver' in PUBLIC_VARS.keys():
            self.receiver = PUBLIC_VARS['receiver']
        else:
            self.receiver = '807447323@qq.com'
        if 'smtpserver' in PUBLIC_VARS.keys():
            self.smtpserver = PUBLIC_VARS['smtpserver']
        else:
            self.smtpserver = 'smtp.qq.com'
        if 'smtp_port' in PUBLIC_VARS.keys():
            self.smtp_port = PUBLIC_VARS['smtp_port']
        else:
            self.smtp_port = 25
        if 'username' in PUBLIC_VARS.keys():
            self.username = PUBLIC_VARS['username']
        else:
            self.username = 'username'
        if 'password' in PUBLIC_VARS.keys():
            self.password = PUBLIC_VARS['password']
        else:
            self.password = 'password'
        if 'subject' in PUBLIC_VARS.keys():
            self.subject = PUBLIC_VARS['subject']
        else:
            self.subject = '小白自动化测试报告'
        if 'report' in PUBLIC_VARS.keys():
            self.report = PUBLIC_VARS['report']
        else:
            self.report = 'report.html'

class CASE_TYPE:
    def __init__(self):
        # Separator
        if 'filepytes' in PUBLIC_VARS.keys():
            self.filepytes = PUBLIC_VARS['filepytes']
        else:
            self.filepytes = [('TEXT', '*.txt'), ('CSV', '*.csv'), ('JSON', '*.json'), ('YAML', '*.yaml')]
        if 'caseSeparator' in PUBLIC_VARS.keys():
            self.caseSeparator = PUBLIC_VARS['caseSeparator']
        else:
            self.caseSeparator = ';;'


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test_Manzhai_APP_Case2.py'
__create_time__ = '2020/7/17 2:29'

from appium import webdriver
from xiaobaiauto2.xiaobaiauto2 import app_aciton
from xiaobaiauto2.utils.xiaobaiauto2Email import send_email
from xiaobaiauto2.data.GLO_VARS import PUBLIC_VARS
from xiaobaiauto2.config.config import EMAILCONFIG
from os import path
from time import sleep
import pytest

browser = None

def setup_module():
    global b
    '''
        'platformName': 'Android',
        'platformVersion': '5.1',  
        'deviceName': '',  
        'noReset': True,  
        'allowClearUserData' = 'true',      #用户可自行选择清除数据
        'fullReset' = "false",              #卸载程序，默认为false
        'exported'="true",                  #是否支持其他应用调用当前组件
        'appPackage': '',  
        'appActivity': '',  
        'unicodeKeyboard': True,            #使用unicode编码方式发送字符串  
        'resetKeyboard': True               #将键盘隐藏起来，输入中文就要增加这两个参数
        'udid':'设备UDID'
        'bundleId':'应用包名'
    '''
    android_caps = {
        'platformName': 'Android',
        'platformVersion': '5.1',
        'deviceName': '设备名',
        'noReset': True,
        'allowClearUserData': 'true',
        'fullReset': "false",
        'exported': "true",
        'appPackage': '应用包名',
        'appActivity': '应用Activity名',
        'unicodeKeyboard': True,
        'resetKeyboard': True
    }
    ios_caps = {
        'platformName': 'iOS',
        'platformVersion': '11.4',
        'deviceName': '设备名',
        'udid': '设备UDID',
        'bundleId': '应用包名',
        'noReset': True,
    }
    emil = {
        'sender': 'tser@jicaiyunshang.com',
        'receiver': ['807447312@qq.com', 'qiankuny@163.com', '912194099@qq.com'],
        'smtpserver': 'smtp.ym.163.com',
        'smtp_port': 25,
        'username': 'tser@jicaiyunshang.com',
        'password': '',
        'subject': '吉彩云尚自动化测试报告',
        'report': 'report.html'
    }
    PUBLIC_VARS.update(emil)
    if b is None:
        b = webdriver.Remote('http://127.0.0.1:4723/wd/hub', android_caps)

def teardown_module():
    b.quit()
    sleep(2)
    _emil = EMAILCONFIG()
    _cur_path = path.abspath(path.curdir)
    if 'report' in PUBLIC_VARS.keys() and '' != PUBLIC_VARS['report']:
        if path.isfile(_cur_path + '/' + PUBLIC_VARS['report']):
            send_email(_cur_path + '/' + PUBLIC_VARS['report'])
    elif '' != _emil.report:
        if path.isfile(_cur_path + '/' + _emil.report):
            send_email(_cur_path + '/' + _emil.report)

def test_yewu_a():
    app_aciton(b, cmd='点击', loc='xxx', data='')
    app_aciton(b, cmd='输入', loc='xxx', data='xxx')

def test_yewu_b():
    app_aciton(b, cmd='点击', loc='xxx', data='')
    app_aciton(b, cmd='输入', loc='xxx', data='xxx')
    app_aciton(b, cmd='属性', contains_assert='吉彩')


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'chinese_chromedriver_installer.py'
__create_time__ = '2020/7/17 1:04'

from typing import Optional
from urllib.request import urlretrieve, urlopen
from urllib.error import URLError
try:
    # windows os
    from winreg import *
except ImportError as e:
    # *unix/mac and so on
    pass
from re import findall, IGNORECASE
from zipfile import ZipFile
from os import remove, path, popen, rename
import platform, sys, subprocess, ssl, tarfile, ctypes

ssl._create_default_https_context = ssl._create_unverified_context

download_filename = ''

def back(a,b,c):
    '''''回调函数
    @a:已经下载的数据块
    @b:数据块的大小
    @c:远程文件的大小
    '''
    per = 100.0*a*b/c
    if per > 100:
        per = 100
    print(end='\r')
    print(f'{download_filename}已下载%.2f%%' % per, end='')

def get_osname():
    ''' 判断当前系统 : windows/mac/ubuntu/centos '''
    if sys.platform == 'win32':
        ''' windows '''
        return 'windows'
    elif sys.platform == 'darwin':
        ''' Mac OS '''
        return 'mac'
    elif sys.platform == 'linux2':
        if 'ubuntu' in platform.platform().lower():
            ''' ubuntu '''
            return 'ubuntu'
        elif 'redhat' in platform.platform().lower():
            ''' redhat/centos '''
            return 'centos'
        else:
            return 'other'
    else:
        return 'other'

def is_admin():
    ''' 适用于windwos 判断当前是否为管理员 return True/False '''
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False

def unZip(filename: Optional[str] = None, newfilename: Optional[str] = None,
          newdirname: Optional[str] = None, dst_path: Optional[str] = '.', **kwargs):
    '''
    解压文件 支持：*.zip *.tar.gz
    :param filename:        待解压文件
    :param newfilename:    解压后的文件名（适应与单文件）
    :param newdirname:     解压后文件夹名(重命名文件夹)
    :param dst_path:    解压的目录
    :param kwargs       扩展参数
    :return:
    样例：
        unZip(filename='chromedriver.zip', dst_path='C:/Windows')
        unZip(filename='chromedriver.tar.gz', newfilename='chromedriver.exe')
    '''
    if dst_path not in [None, ''] and newfilename in [None, '']:
        if '.zip' == filename[-4:]:
            first_name = ZipFile(f"{path.abspath(path.curdir)}/{download_filename}").namelist()[0].replace('/', '')
            ZipFile(filename).extractall(path=dst_path)
        elif '.tar.gz' == filename[-7:]:
            first_name = tarfile.open(f"{path.abspath(path.curdir)}/{download_filename}").getnames()[0].replace('/', '')
            tarfile.open(filename).extractall(path=dst_path)
        else:
            first_name = tarfile.open(f"{path.abspath(path.curdir)}/{download_filename}").getnames()[0].replace('/', '')
            tarfile.open(filename).extractall(path=dst_path)
        if path.isdir(f'{path.abspath(dst_path)}/{first_name}'):
            rename(src=f'{path.abspath(dst_path)}/{first_name}', dst=f'{path.abspath(dst_path)}/{newdirname}')
    elif path in [None, ''] and newfilename not in [None, '']:
        if '.zip' == filename[-4:]:
            ZipFile(filename).extract(newfilename)
        elif '.tar.gz' == filename[-7:]:
            tarfile.open(filename).extract(newfilename)

def download(url: Optional[str] = None, **kwargs):
    '''
    下载文件
    :param url:          下载链接
    :param filename:     下载文件名
    :param newfilename:  解压后文件名
    :param newdirname:   解压后文件夹名
    :param dst_path:     (解压)/(移动)的目录
    :param unzip:        是否解压
    :param del_src:      是否删除下载文件
    :param kwargs:
    :return:
    样例：
        download(url='https://api.xiaobai.com/adb.zip', filename='adb.zip', dst_path='D:/adb', unzip=True, del_src=True)
    '''
    global download_filename
    if 'filename' in kwargs.keys():
        download_filename = kwargs.get('filename')
    else:
        download_filename = ''
    if url not in [None, '']:
        try:
            if 'unzip' not in kwargs.keys() and 'dst_path' in kwargs.keys():
                urlretrieve(url, kwargs.get('dst_path') + '/' + download_filename, back)
            else:
                urlretrieve(url, download_filename, back)
        except URLError as e:
            print('您的文件下载失败，请确认后重试！')
            exit(0)
        except ConnectionResetError as e:
            print('您的文件下载失败，网络连接重试失败！')
            exit(0)
        except KeyboardInterrupt as e:
            print('您已主动取消了下载！')
            exit(0)
        if 'unzip' in kwargs.keys() and kwargs.get('unzip'):
            if 'newfilename' in kwargs.keys():
                unZip(filename=download_filename, newfilename=kwargs.get('newfilename'))
            elif 'dst_path' in kwargs.keys() and 'newdirname' in kwargs.keys():
                unZip(filename=download_filename, newdirname=kwargs.get('newdirname'), dst_path=kwargs.get('dst_path'))
            else:
                unZip(filename=download_filename, dst_path=kwargs.get('dst_path'))
        if 'del_src' in kwargs.keys() and kwargs.get('del_src'):
            remove(download_filename)

def _shell(cmd: Optional[str] = '', isSubprocess: Optional[bool] = True, powershell: Optional[bool] = False):
    ''' subprocess '''
    name = get_osname()
    if isSubprocess:
        if powershell:
            if cmd == '' and name == 'windows' and powershell:
                args = ["powershell",
                        "Set-ExecutionPolicy RemoteSigned -scope CurrentUser",
                        "iex (new-object net.webclient).downloadstring('https://get.scoop.sh')",
                        "iwr -useb get.scoop.sh | iex"]
            else:
                args = ['powershell', cmd]
        else:
            cmd_list = cmd.split(' ')
            args = [cmd_list[0], ' '.join(cmd_list[1:])]
        try:
            sp = subprocess.Popen(args, stdout=subprocess.PIPE)
            return sp.stdout.read()
        except Exception as e:
            return False
    else:
        try:
            popen(cmd)
        except Exception as e:
            exit(0)

def setEnv(name: Optional[str] = None, value: Optional[str] = None,
           addPath: Optional[bool] = True, endPath: Optional[str] = '/bin'):
    '''
    添加(系统)环境变量
    :param name:        环境变量名
    :param value:       环境变量值
    :param addPath:     是否添加到PATH
    :param endPath:     追加的路径
    :return:
    样例：
        setEnv(JAVA_HOME, 'D:/JDK', True, '/bin')
        setEnv(JAVA_HOME, 'D:/JDK')
    '''
    if name is None or value is None: return None
    os = get_osname()
    if os == 'windows':
        value = value.replace('/', '\\')
        endPath = endPath.replace('/', '\\')
        if is_admin():
            env = '系统'
            read_env_key = OpenKey(HKEY_LOCAL_MACHINE, 'SYSTEM\ControlSet001\Control\Session Manager\Environment')
            write_env_key = OpenKey(HKEY_LOCAL_MACHINE, 'SYSTEM\ControlSet001\Control\Session Manager\Environment',
                              access=KEY_WRITE)
        else:
            env = '用户'
            read_env_key = OpenKey(HKEY_CURRENT_USER, 'Environment')
            write_env_key = OpenKey(HKEY_CURRENT_USER, 'Environment', access=KEY_WRITE)
        path_env = QueryValueEx(read_env_key, 'path')[0]
        # add key
        SetValueEx(write_env_key, name, 0, REG_SZ, value)
        if addPath:
            SetValueEx(write_env_key, 'path', 0, REG_EXPAND_SZ, f'%{name}%{endPath};{path_env}')
        # FlushKey(write_env_key)
        write_env_key.Close()
        print(f'\n{env}环境变量{name}已经设置完毕！')
    else:
        cmd = f'\nexport {name}={value}'
        if addPath:
            cmd += f'\nexport PATH=$PATH:${name}{endPath}'
        try:
            with open('/etc/profile', 'a') as f:
                f.write(cmd)
                f.flush()
                f.close()
            _shell('source /etc/profile')
            print(f'\n环境变量 {name} 已经设置完毕！')
        except PermissionError as e:
            print('\n未使用管理员权限')
            try:
                _shell('cd ~')
                if not path.isfile('.bash_profile'):
                    _shell('touch .bash_profile')
                with open('.bash_profile', 'a') as f:
                    f.write(cmd)
                    f.flush()
                    f.close()
                _shell('source /etc/profile')
                print(f'\n用户环境变量 {name} 已经设置完毕！')
            except Exception as e:
                print('\n环境变量设置失败！')

def getWebList(url: Optional[str] = None, match: Optional[str] = None, flags: Optional[int] = IGNORECASE):
    '''
    获取网页中指定规则的所有值，存储到列表中，默认
    :param url:     链接地址
    :param match:   匹配规则
    :param flags:   匹配模式 默认(IGNORECASE)忽略大小写
    :return:
    样例：
        getWebList('https://www.baidu.com', 'href="(.+?)"')
    '''
    if url is not None and match is not None:
        try:
            res = urlopen(url=url).read().decode('utf-8')
            return findall(match, res, flags)
        except URLError as e:
            print(e)
            exit(0)
    else:
        return []

def chromedriver_download(version: Optional[str] = '100', dest_dir: Optional[str] = '.'):
    '''
    自动下载chromedriver驱动文件
    :param version: 87,88等chrome浏览器版本号（整数）
    :return:
    '''
    TAOBAO_MIRROR_DOWNLOAD_URL = 'https://npm.taobao.org/mirrors/chromedriver/'
    name = get_osname()
    if name == 'windows':
        try:
            if version == '':
                chrome_current_version = EnumValue(OpenKey(HKEY_CURRENT_USER, 'Software\Google\Chrome\BLBeacon'), 0)[1]
                cur_version = chrome_current_version.split('.')[0]
            else:
                cur_version = version
            LAST_VERSION = getWebList(TAOBAO_MIRROR_DOWNLOAD_URL, f'/mirrors/chromedriver/{cur_version}([0-9\.]+)/">')[0]
            DOWNLOAD_URL = f'{TAOBAO_MIRROR_DOWNLOAD_URL}{cur_version}{LAST_VERSION}/chromedriver_win32.zip'
            download(DOWNLOAD_URL, filename='chromedriver_win32.zip', dst_path=dest_dir, unzip=True, del_src=True)
        except Exception as e:
            raise ('是不是没安装Chrome浏览器呢？' + str(e))
    elif name == 'mac':
        if version == '':
            cur_version = ''
        else:
            cur_version = version
        try:
            LAST_VERSION = getWebList(TAOBAO_MIRROR_DOWNLOAD_URL, f'/mirrors/chromedriver/{cur_version}([0-9\.]+)/">')[0]
            DOWNLOAD_URL = f'{TAOBAO_MIRROR_DOWNLOAD_URL}{LAST_VERSION}/chromedriver_linux64.zip'
            download(DOWNLOAD_URL, filename='chromedriver_linux64.zip', dst_path='/usr/local/bin', unzip=True, del_src=True)
        except Exception as e:
            pass
    else:
        if version == '':
            cur_version = ''
        else:
            cur_version = version
        try:
            LAST_VERSION = getWebList(TAOBAO_MIRROR_DOWNLOAD_URL, f'/mirrors/chromedriver/{cur_version}([0-9\.]+)/">')[0]
            DOWNLOAD_URL = f'{TAOBAO_MIRROR_DOWNLOAD_URL}{LAST_VERSION}/chromedriver_mac64.zip'
            download(DOWNLOAD_URL, filename='chromedriver_mac64.zip', dst_path='/usr/local/bin', unzip=True, del_src=True)
        except Exception as e:
            pass

def tesseract_download():
    TESSERACT_DOWNLOAD_URL = 'https://api.256file.com/download/56476_tesseract.exe'
    download(url=TESSERACT_DOWNLOAD_URL, filename='tesseract.exe', unzip=False)

def update_keyword_db():
    KEYWORD_DB_DOWNLOAD_URL = ''
    urlretrieve(KEYWORD_DB_DOWNLOAD_URL, 'xiaobaiauto2.db', back)
    # popen('xiaobaiauto2.db', )

def jdk_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    '''
    下载并安装openjdk
    windows: scoop install oraclejdk{version}
    mac os: brew cask install adoptopenjdk{version}
    ubuntu: apt install openjdk-{version}-jre
    centos: yum install java-1.{version}.0-openjdk
    mirror: https://mirrors.tuna.tsinghua.edu.cn/AdoptOpenJDK/{version}/jdk/x64/{os}/
    '''
    latest = getWebList(url='https://mirrors.tuna.tsinghua.edu.cn/AdoptOpenJDK/', match='title="[0-9]+"')
    version = latest[0] if len(latest)>0 and version == 'latest' else version
    os = get_osname()
    if os == 'windows':
        url = f'https://mirrors.tuna.tsinghua.edu.cn/AdoptOpenJDK/{version}/jdk/x64/{os}/'
        filename = getWebList(url, match='href="OpenJDK(.+?).zip"', flags=0)
        if len(filename) > 0:
            url = f'{url}OpenJDK{filename[0]}.zip'
            download(url=url, filename=f'jdk{version}_{os}_64.zip', newdirname=f'jdk{version}_{os}_64', dst_path=dest_dir, unzip=True, del_src=True)
        setEnv('JAVA_HOME', f'{path.abspath(dest_dir)}/jdk{version}_{os}_64')
    elif os == 'mac':
        ''' Mac OS '''
        try:
            _shell(cmd='/bin/zsh -c "$(curl -fsSL https://gitee.com/cunkai/HomebrewCN/raw/master/Homebrew.sh)"')
        except Exception as e:
            pass
        url = f'https://mirrors.tuna.tsinghua.edu.cn/AdoptOpenJDK/{version}/jdk/x64/{os}/'
        filename = getWebList(url, match='href="OpenJDK(.+?).tar.gz"', flags=0)
        if len(filename) > 0:
            url = f'{url}OpenJDK{filename[0]}.tar.gz'
            download(url=url, filename=f'jdk{version}_{os}_64.tar.gz', newdirname=f'jdk{version}_{os}_64', dst_path=dest_dir, unzip=True, del_src=True)
        setEnv('JAVA_HOME', f'{path.abspath(dest_dir)}/jdk{version}_{os}_64/Contents/Home')
    else:
        url = f'https://mirrors.tuna.tsinghua.edu.cn/AdoptOpenJDK/{version}/jdk/x64/linux/'
        filename = getWebList(url, match='href="OpenJDK(.+?).tar.gz"', flags=0)
        if len(filename) > 0:
            url = f'{url}OpenJDK{filename[0]}.tar.gz'
            download(url=url, filename=f'jdk{version}_{os}_64.tar.gz', newdirname=f'jdk{version}_{os}_64', dst_path=dest_dir, unzip=True, del_src=True)
        setEnv('JAVA_HOME', f'{path.abspath(dest_dir)}/jdk{version}_{os}_64/Contents/Home')

def maven_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    '''
    下载并安装maven
    '''
    latest = getWebList(url='http://maven.apache.org/download.cgi', match='apache-maven-(.+?)-bin.zip')
    version = latest[0] if len(latest)>0 and version == 'latest' else version
    os = get_osname()
    url = f'https://mirrors.tuna.tsinghua.edu.cn/apache/maven/maven-{version.split(".")[0]}/{version}/binaries/apache-maven-{version}-bin.zip'
    download(url=url, filename=f'maven{version}_{os}_64.zip', newdirname=f'maven{version}_{os}_64', dst_path=dest_dir, unzip=True, del_src=True)
    setEnv('MAVEN_HOME', f'{path.abspath(dest_dir)}/maven{version}_{os}_64')

def jmeter_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    ''' 下载并配置环境变量 '''
    src_list = [
        'https://mirror.bit.edu.cn/apache/jmeter/binaries/',
        'https://mirrors.bfsu.edu.cn/apache/jmeter/binaries/',
        'https://mirrors.tuna.tsinghua.edu.cn/apache/jmeter/binaries/'
    ]
    latest = getWebList(url='https://mirrors.tuna.tsinghua.edu.cn/apache/jmeter/binaries/HEADER.html', match='Apache JMeter (.+?) requires')
    version = latest[0] if len(latest) > 0 and version == 'latest' else version
    filename = f'apache-jmeter-{version}.zip'
    download(src_list[2] + filename, filename=filename, newdirname=f'apache-jmeter-{version}', dst_path=dest_dir, unzip=True, del_src=True)
    setEnv('JMETER_HOME', f'{path.abspath(dest_dir)}/apache-jmeter-{version}')
    # add plugin manager
    plugin_latest = getWebList(url='https://repo1.maven.org/maven2/kg/apc/jmeter-plugins-manager/maven-metadata.xml',
                               match='<latest>(.+?)</latest>')
    download(url=f'https://repo1.maven.org/maven2/kg/apc/jmeter-plugins-manager/{plugin_latest[0]}/jmeter-plugins-manager-{plugin_latest[0]}.jar',
             filename=f'jmeter-plugins-manager-{plugin_latest[0]}.jar',
             dst_path=f'{path.abspath(dest_dir)}/apache-jmeter-{version}/lib/ext')
    # set Chinese
    with open(path.abspath(dest_dir) + f'/apache-jmeter-{version}/bin/jmeter.properties', 'a', encoding='utf-8') as fw:
        fw.write('\r\n\r\nlanguage=zh_CN')
        fw.write('\r\nsampleresult.default.encoding=UTF-8')
        fw.flush()
        fw.close()
    # print('JMeter已设置中文，请求结果编码已设置为UTF-8')

def tomcat_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    if 'latest' == version:
        latest0 = getWebList(url='https://mirrors.tuna.tsinghua.edu.cn/apache/tomcat/', match='tomcat-(\d+)/')
        V = str(max([int(i) for i in latest0]))
    elif '.' not in version:
        V = version
    else:
        V = version.split(".")[0]
    latest = getWebList(url=f'https://mirrors.tuna.tsinghua.edu.cn/apache/tomcat/tomcat-{V}/',
                        match=f'v([0-9a-zA-Z\-\.]+)/')
    version = latest[-1] if len(latest) > 0 and version.split(".")[0] == V else version
    url = f'https://mirrors.tuna.tsinghua.edu.cn/apache/tomcat/tomcat-{version.split(".")[0]}/v{version}/bin/apache-tomcat-{version}.zip'
    download(url=url, filename=f'tomcat_{version}.zip', dst_path=dest_dir, unzip=True, del_src=True)
    setEnv(name='CATALINA_HOME', value=f'{path.abspath(dest_dir)}/tomcat_{version}',
           endPath='\lib;%CATALINA_HOME%\lib\servlet-api.jar;%CATALINA_HOME%\lib\jsp-api.jar')

def jenkins_war_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    url = f'https://mirrors.tuna.tsinghua.edu.cn/jenkins/war-stable/{version}/jenkins.war'
    download(url=url, filename='jenkins.war', dst_path=dest_dir)

def git_windows_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    if version == 'latest':
        if get_osname() == 'windows':
            url = 'https://mirrors.tuna.tsinghua.edu.cn/github-release/git-for-windows/git/LatestRelease/'
            files = getWebList(url=url, match='href="Git-(.+?)-64-bit.exe"')
            download(url=f'{url}Git-{files[0]}-64-bit.exe', filename=f'Git-{files[0]}-64-bit.exe', dst_path=dest_dir)
        else:
            print('非windows系统可以使用命令自行安装哦！')
    else:
        print('该程序暂不持支选择版本哦！')

def node_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    name = get_osname()
    res = urlopen(url='https://npm.taobao.org/mirrors/node/index.tab').read().decode('utf-8')
    all_lines = res.split('\n')
    if len(all_lines) > 1:
        if version == 'latest':
            version = all_lines[1].split('\t')[0][1:]
        else:
            version = [v.split('\t')[0][1:] for v in all_lines if f'v{version}' in v.split('\t')[0]]
            if len(version) == 0:
                version = all_lines[1].split('\t')[0][1:]
        if name == 'windows':
            url = f'https://npm.taobao.org/mirrors/node/v{version}/node-v{version}-win-x64.zip'
            download(url=url, filename=f'node-v{version}-win-x64.zip', dst_path=dest_dir, unzip=True, del_src=True)
            setEnv(name='NODE_HOME', value=f'{path.abspath(dest_dir)}/node-v{version}-win-x64', endPath='')
        elif name == 'mac':
            url = f'https://npm.taobao.org/mirrors/node/v{version}/node-v{version}-darwin-x64.tar.gz'
            download(url=url, filename=f'node-v{version}-darwin-x64.tar.gz', dst_path=dest_dir, unzip=True, del_src=True)
            setEnv(name='NODE_HOME', value=f'{path.abspath(dest_dir)}/node-v{version}-darwin-x64', endPath='')
        else:
            url = f'https://npm.taobao.org/mirrors/node/v{version}/node-v{version}-linux-x64.tar.gz'
            download(url=url, filename=f'node-v{version}-linux-x64.tar.gz', dst_path=dest_dir, unzip=True, del_src=True)
            setEnv(name='NODE_HOME', value=f'{path.abspath(dest_dir)}/node-v{version}-linux-x64', endPath='')

def allure_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    ''' 默认安装开源版 '''
    url = 'https://repo.maven.apache.org/maven2/io/qameta/allure/allure-commandline/maven-metadata.xml'
    # url = 'https://github.com/allure-framework/allure2/releases'
    latest = getWebList(url=url, match='<latest>(.+?)</latest>')
    version = latest[0] if version == 'latest' and len(latest) > 0 else version
    name = get_osname()
    if name == 'windows':
        download(url=f'https://repo.maven.apache.org/maven2/io/qameta/allure/allure-commandline/{version}/allure-commandline-{version}.zip',
                 filename=f'allure-{version}.zip', dst_dir=dest_dir, unzip=True, del_src=False)
    else:
        download(url=f'https://repo.maven.apache.org/maven2/io/qameta/allure/allure-commandline/{version}/allure-commandline-{version}.tgz',
                 filename=f'allure-{version}.tgz', dst_dir=dest_dir, unzip=True, del_src=False)
    setEnv(name='ALLURE_HOME', value=f'{path.abspath(dest_dir)}/allure-{version}', addPath=True)

def svn_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    ''' SVN安装 '''
    if version == 'latest':
        url = 'https://mirrors.tuna.tsinghua.edu.cn/apache/subversion'
        latest = getWebList(url=url,
                            match='href="https://osdn.net/projects/tortoisesvn/storage/(.+?)/Application/TortoiseSVN-(.+?)-x64-svn-(.+?).msi/"')
        if len(latest) > 0:
            download(
                url=f'https://osdn.net/projects/tortoisesvn/storage/{latest[0][0]}/Application/TortoiseSVN-{latest[0][1]}-x64-svn-{latest[0][2]}.msi"',
                dst_dir=dest_dir
            )
    else:
        url = f'https://osdn.net/projects/tortoisesvn/storage/{version}/Application/'
        latest = getWebList(url=url, match=f'href="/projects/tortoisesvn/storage/1.14.0/Application/TortoiseSVN-(.+?)-x64-svn-{version}.msi"')
        if len(latest) > 0:
            download(
                url=f'https://osdn.net/projects/tortoisesvn/storage/{version}/Application/TortoiseSVN-{latest[0]}-x64-svn-{version}.msi/"',
                dst_path=dest_dir
            )

def adb_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    if version != 'latest': return
    name = get_osname()
    if name == 'windows':
        try:
            download(url='https://dl.google.com/android/repository/platform-tools-latest-windows.zip',
                     filename=f'platform-tools-latest-windows.zip', newdirname='platform-tools',
                     dst_path=dest_dir, unzip=True, del_src=True)
        except Exception as e:
            download(url='https://dl.adbdriver.com/upload/ADBDriverInstaller.exe', filename='ADBDriverInstaller.exe',
                     dst_path=dest_dir)
    elif name == 'mac':
        download(url='https://dl.google.com/android/repository/platform-tools-latest-darwin.zip',
                 filename=f'platform-tools-latest-darwin.zip', newdirname='platform-tools',
                 dst_path=dest_dir, unzip=True, del_src=True)
    else:
        download(url='https://dl.google.com/android/repository/platform-tools-latest-linux.zip',
                 filename=f'platform-tools-latest-linux.zip', newdirname='platform-tools',
                 dst_path=dest_dir, unzip=True, del_src=True)
    setEnv(name='ADB_HOME', value=f'{path.abspath(dest_dir)}/platform-tools', addPath=True)

def fiddler_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    name = get_osname()
    if version != 'latest': return
    if name == 'windows':
        download(url='https://telerik-fiddler.s3.amazonaws.com/fiddler/FiddlerSetup.exe', filename='FiddlerSetup.exe',
                 dst_path=dest_dir)
        download(url='https://www.telerik.com/download/fiddler/fiddler-everywhere-windows', filename='fiddler-everywhere.exe',
                 dst_path=dest_dir)
    elif name == 'mac':
        download(url='https://www.telerik.com/download/fiddler/fiddler-everywhere-osx', filename='fiddler-everywhere.dmg',
                 dst_path=dest_dir)
    else:
        download(url='https://www.telerik.com/download/fiddler/fiddler-everywhere-linux', filename='fiddler-everywhere.AppImage',
                 dst_path=dest_dir)

def postman_install(version: Optional[str] = 'latest', dest_dir: Optional[str] = '.'):
    if version != 'latest': return
    name = get_osname()
    if name == 'windows':
        download(url='https://dl.pstmn.io/download/latest/win64', filename='postman-latest-win64.exe', dst_path=dest_dir)
    elif name == 'mac':
        download(url='https://dl.pstmn.io/download/latest/osx', filename='postman-osx-latest.zip', dst_path=dest_dir,
                 unzip=True, del_src=True)
    else:
        download(url='https://dl.pstmn.io/download/latest/linux64', filename='postman-linux64-latest.tar.gz',
                 dst_path=dest_dir, unzip=True, del_src=True)

def install(download_url: Optional[str] = None, **kwargs):
    '''
    指定开源软件
    :param download_url:    str,下载地址或多版本下载页
    :param version:         str,指定版本，默认最新版
    :param match_version:   str,模糊匹配版本，正则匹配，默认匹配指定版本或最大版本
    :param unzip:           bool,是否解压缩包，默认解压
    :param filename:        str,下载文件名
    :param newfilename:     str,解压后文件名
    :param newdirname:      str,解压后文件夹名
    :param env_name:        str,设置环境变量名
    :param env_value:       str,设置环境变量值
    :param addPath:         bool,是否添加到path变量中
    :param kwargs:          待扩展功能
    :return:
    '''
    if 'version' in kwargs.keys():
        version = kwargs.get('version')
    else:
        version = 'latest'
    if 'match_version' in kwargs.keys():
        match_version = kwargs.get('match_version')
    else:
        match_version = None
    if 'unzip' in kwargs.keys():
        unzip = kwargs.get('unzip')
    else:
        unzip = False
    if 'env_name' in kwargs.keys():
        env_name = kwargs.get('env_name')
    else:
        env_name = None
    if 'env_value' in kwargs.keys():
        env_value = kwargs.get('env_value')
    else:
        env_value = None
    if 'addPath' in kwargs.keys():
        addPath = kwargs.get('addPath')
    else:
        addPath = False


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'xiaobaiauto2db.py'
__create_time__ = '2020/7/2 21:45'

from sqlite3 import connect
from xiaobaiauto2.config.config import DBCONFIG

class DB(object):
    def __init__(self):
        self.conn = connect(DBCONFIG().dbpath)
        self.cur = self.conn.cursor()

    def select(self, sql=None, parames=None, *args, **kwargs):
        return self.cur.execute(sql).fetchall()

    def update(self, sql=None, parames=None, *args, **kwargs):
        self.cur.execute(sql)
        self.conn.commit()

    def __del__(self):
        self.cur.close()
        self.conn.close()


#! /usr/bin/env python
__author__ = 'Tser'
__email__ = '807447312@qq.com'
__project__ = 'xiaobaiauto2'
__script__ = 'test03.py'
__create_time__ = '2020/7/17 2:29'

import pytest

def setup_module():
    '''测试用户文件第一个执行的方法'''

def teardown_module():
    '''测试用户文件最后一个执行的方法'''

def test_yewu_a():
    '''正常业务测试方法1'''

def test_yewu_b():
    '''正常业务测试方法2'''